/*
 *  expr/exon.c
 * 
 *  $Author: baptiste $, $Date: 2008-05-13 15:33:52 $, $Version$
 *  
 *  Libgdl : a C library for statistical genetics
 * 
 *  Copyright (C) 2009  Jean-Baptiste Veyrieras, Univeristy of Chicago
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA * 
 */
#include <gdl/gdl_common.h>
#include <gdl/gdl_errno.h>
#include <gdl/gdl_string.h>
#include <gdl/gdl_sort.h>
#include <gdl/gdl_statistics.h>
#include <gdl/gdl_math.h>
#include <gdl/gdl_rng.h>
#include <gdl/gdl_randist.h>
#include <gdl/gdl_specfunc.h>
#include <gdl/gdl_sort_double.h>
#include <gdl/gdl_statistics_double.h>
#include <gdl/gdl_expr_exon.h>

gdl_expr_exon *
gdl_expr_exon_alloc (const size_t start, const size_t end)
{
	gdl_expr_exon * p;
	
	p = GDL_CALLOC (gdl_expr_exon, 1);
	
	p->start  = start;
	p->end    = end;
	
	return p;
}

void
gdl_expr_exon_free (gdl_expr_exon * p)
{
	if (p)
	{
		GDL_FREE (p->tx_idx);
		GDL_FREE (p);
	}
}


gdl_expr_exon *
gdl_expr_exon_fread (FILE * stream)
{
	if (stream)
	{
		int status;
		gdl_expr_exon * p;

		p = GDL_CALLOC (gdl_expr_exon, 1);
		
		status = fread(&(p->start), sizeof(size_t), 1, stream);
		GDL_FREAD_STATUS (status, 1);
		status = fread(&(p->end), sizeof(size_t), 1, stream);
		GDL_FREAD_STATUS (status, 1);
		status = fread(&(p->ntx), sizeof(size_t), 1, stream);
		GDL_FREAD_STATUS (status, 1);
		
		if (p->ntx)
		{
			p->tx_idx = GDL_MALLOC (size_t, p->ntx);
			status = fread(p->tx_idx, sizeof(size_t), p->ntx, stream);
			GDL_FREAD_STATUS (status, p->ntx);
		}
				
		return p;
	}
	return 0;	
}

int
gdl_expr_exon_fwrite (FILE * stream, const gdl_expr_exon * p)
{
	if (stream && p)
	{
		int status;
		
		status = fwrite(&(p->start), sizeof(size_t), 1, stream);
		GDL_FWRITE_STATUS (status, 1);
		status = fwrite(&(p->end), sizeof(size_t), 1, stream);
		GDL_FWRITE_STATUS (status, 1);
		status = fwrite(&(p->ntx), sizeof(size_t), 1, stream);
		GDL_FWRITE_STATUS (status, 1);
		if (p->ntx)
		{
			status = fwrite(p->tx_idx, sizeof(size_t), p->ntx, stream);
			GDL_FWRITE_STATUS (status, p->ntx);
		}
				
		return GDL_SUCCESS;
	}
	return GDL_EINVAL;
}

size_t 
gdl_expr_exon_add_tx (gdl_expr_exon * e, const size_t tx)
{
	if (e->ntx)
	{
		size_t * tmp = GDL_MALLOC (size_t, e->ntx + 1);
		memcpy(tmp, e->tx_idx, sizeof(size_t)*e->ntx);
		GDL_FREE (e->tx_idx);
		e->tx_idx = tmp;
	}
	else
	{
		e->tx_idx = GDL_MALLOC (size_t, 1);	
	}
	e->tx_idx[e->ntx] = tx;
	return ++(e->ntx);
}
