/*
 *  igdb/annot.c
 *
 *  $Author: baptiste $, $Date: 2008-05-13 15:33:43 $, $Version$
 *
 *  eQTNMiner : a C package for high-resolution mapping of eQTNs
 *
 *  Copyright (C) 2008  Jean-Baptiste Veyrieras, University of Chicago, USA.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 */


#include <math.h>

#include <gdl/gdl_common.h>
#include <gdl/gdl_version.h>
#include <gdl/gdl_errno.h>
#include <gdl/gdl_io.h>
#include <gdl/gdl_math.h>
#include <gdl/gdl_string.h>
#include <gdl/gdl_expr_feature.h>
#include <gdl/gdl_expr_feature_anchor.h>
#include <gdl/gdl_igdb_chromosome.h>
#include <gdl/gdl_igdb_genome.h>
#include <gdl/gdl_igdb_cis_annot.h>

gdl_igdb_cis_annot_entry *
gdl_igdb_cis_annot_entry_alloc (void)
{
	gdl_igdb_cis_annot_entry * e;

	e = GDL_CALLOC (gdl_igdb_cis_annot_entry, 1);

	return e;
}

void
gdl_igdb_cis_annot_entry_free (gdl_igdb_cis_annot_entry * e)
{
	if (e)
	{
		GDL_FREE (e);
	}
}

gdl_igdb_cis_annot_chromosome *
gdl_igdb_cis_annot_chromosome_alloc (void)
{
	gdl_igdb_cis_annot_chromosome * a;

	a = GDL_CALLOC (gdl_igdb_cis_annot_chromosome, 1);

	return a;
}

void
gdl_igdb_cis_annot_chromosome_free (gdl_igdb_cis_annot_chromosome * a)
{
	if (a)
	{
		size_t i;
		for (i = 0; i < a->size; i++)
		{
			gdl_igdb_cis_annot_entry_free (a->entries[i]);
		}
		GDL_FREE (a->entries);
		GDL_FREE (a);
	}
}

gdl_igdb_cis_annot_chromosome *
gdl_igdb_cis_annot_chromosome_fscanf2 (FILE * stream)
{
	size_t i,j,n;
	gdl_string * line, * tok;
	gdl_igdb_cis_annot_entry * entry;
	gdl_igdb_cis_annot_chromosome * annot;
	gdl_list * entries = gdl_list_alloc (gdl_list_default);

	annot = gdl_igdb_cis_annot_chromosome_alloc ();

	line=0;
	while(gdl_getline (&line, &n, stream) != -1)
	{
		i=j=0;
		entry = gdl_igdb_cis_annot_entry_alloc ();
		tok = gdl_string_next_token (line, n, &i, &j);
		gdl_string_free (tok);
		tok = gdl_string_next_token (line, n, &i, &j);
		entry->start = entry->end = atol(tok);
		gdl_string_free (tok);
		gdl_list_push_back (entries, entry, 0);
		gdl_string_free (line);
		line=0;
	}

	annot->size    = gdl_list_size (entries);
	annot->entries = GDL_MALLOC (gdl_igdb_cis_annot_entry *, annot->size);
	i=0;
	gdl_list_itr * itr = gdl_list_iterator_front (entries);

	do
	{
		annot->entries[i] = gdl_list_iterator_value (itr);
		i++;
	}
	while (gdl_list_iterator_next (itr));

	gdl_list_iterator_free (itr);
	gdl_list_free (entries);

	return annot;
}

gdl_igdb_cis_annot_chromosome *
gdl_igdb_cis_annot_chromosome_fscanf3 (FILE * stream)
{
	size_t i,j,n;
	gdl_string * line, * tok;
	gdl_igdb_cis_annot_entry * entry;
	gdl_igdb_cis_annot_chromosome * annot;
	gdl_list * entries = gdl_list_alloc (gdl_list_default);

	annot = gdl_igdb_cis_annot_chromosome_alloc ();

	line=0;
	while(gdl_getline (&line, &n, stream) != -1)
	{
		i=j=0;
		entry = gdl_igdb_cis_annot_entry_alloc ();
		tok = gdl_string_next_token (line, n, &i, &j);
		gdl_string_free (tok);
		tok = gdl_string_next_token (line, n, &i, &j);
		entry->start = atol(tok);
		gdl_string_free (tok);
		tok = gdl_string_next_token (line, n, &i, &j);
		entry->end = atol(tok);
		gdl_string_free (tok);
		gdl_list_push_back (entries, entry, 0);
		gdl_string_free (line);
		line=0;
	}

	annot->size    = gdl_list_size (entries);
	annot->entries = GDL_MALLOC (gdl_igdb_cis_annot_entry *, annot->size);
	i=0;
	gdl_list_itr * itr = gdl_list_iterator_front (entries);
	do
	{
		annot->entries[i] = gdl_list_iterator_value (itr);
		i++;
	}
	while (gdl_list_iterator_next (itr));
	gdl_list_iterator_free (itr);
	gdl_list_free (entries);

	return annot;
}

gdl_igdb_cis_annot_chromosome *
gdl_igdb_cis_annot_chromosome_fscanf4 (FILE * stream)
{
	size_t i,j,n;
	gdl_string * line, * tok;
	gdl_igdb_cis_annot_entry * entry;
	gdl_igdb_cis_annot_chromosome * annot;
	gdl_list * entries = gdl_list_alloc (gdl_list_default);

	annot = gdl_igdb_cis_annot_chromosome_alloc ();

	line=0;
	while(gdl_getline (&line, &n, stream) != -1)
	{
		i=j=0;
		entry = gdl_igdb_cis_annot_entry_alloc ();
		tok = gdl_string_next_token (line, n, &i, &j);
		gdl_string_free (tok);
		tok = gdl_string_next_token (line, n, &i, &j);
		entry->start = atol(tok);
		gdl_string_free (tok);
		tok = gdl_string_next_token (line, n, &i, &j);
		entry->end = atol(tok);
		gdl_string_free (tok);
		tok = gdl_string_next_token (line, n, &i, &j);
		entry->score = (double) atof(tok);
		gdl_string_free (tok);
		gdl_list_push_back (entries, entry, 0);
		gdl_string_free (line);
		line=0;
	}

	annot->size    = gdl_list_size (entries);
	annot->entries = GDL_MALLOC (gdl_igdb_cis_annot_entry *, annot->size);
	i=0;
	gdl_list_itr * itr = gdl_list_iterator_front (entries);
	do
	{
		annot->entries[i] = gdl_list_iterator_value (itr);
		i++;
	}
	while (gdl_list_iterator_next (itr));
	gdl_list_iterator_free (itr);
	gdl_list_free (entries);

	return annot;
}

gdl_igdb_cis_annot_track_type *
gdl_igdb_cis_annot_track_type_alloc (const gdl_igdb_cis_annot_track_type * T)
{
	gdl_igdb_cis_annot_track_type * t;

	t = GDL_CALLOC (gdl_igdb_cis_annot_track_type, 1);

	t->name      = gdl_string_clone (T->name);
	t->load      = T->load;
	t->p_size    = T->p_size;
	t->snp_value = T->snp_value;
	t->gene_only = T->gene_only;
	t->spurious  = T->spurious;
	t->thresh    = T->thresh;

	return t;
}

void
gdl_igdb_cis_annot_track_type_free (gdl_igdb_cis_annot_track_type * t)
{
	if (t)
	{
		gdl_string_free (t->name);
		GDL_FREE (t);
	}
}

gdl_igdb_cis_annot_track *
gdl_igdb_cis_annot_track_alloc (const gdl_igdb_cis_annot_track_type * T, const gdl_string * name, const size_t nthread)
{
	gdl_igdb_cis_annot_track * t;

	t = GDL_CALLOC (gdl_igdb_cis_annot_track, 1);

	t->name = gdl_string_clone (name);
	t->T    = gdl_igdb_cis_annot_track_type_alloc (T);
	t->nthread = (!nthread) ? 1 : nthread;
	t->focusth = GDL_CALLOC (gdl_igdb_cis_annot_chromosome *, nthread);

	return t;
}

void
gdl_igdb_cis_annot_track_mth (gdl_igdb_cis_annot_track * t, const size_t nthread)
{
	if (t)
	{
		GDL_FREE (t->focusth);
		t->nthread = (!nthread) ? 1 : nthread;
		t->focusth = GDL_CALLOC (gdl_igdb_cis_annot_chromosome *, nthread);
	}
}

void
gdl_igdb_cis_annot_track_free (gdl_igdb_cis_annot_track * t)
{
	if (t)
	{
		gdl_string_free (t->name);
		gdl_string_free (t->dir);
		gdl_igdb_cis_annot_track_type_free (t->T);
		gdl_igdb_cis_annot_chromosome_free (t->chrom);
		if (t->nthread)
		{
			GDL_FREE (t->focusth);
		}
		GDL_FREE (t);
	}
}

int
gdl_igdb_cis_annot_track_load (gdl_igdb_cis_annot_track * t, const gdl_igdb_chromosome * chrom)
{
	int status = GDL_FAILURE;

	if (t->T->load)
	{
		gdl_string * filename;
		FILE * stream;

		gdl_igdb_cis_annot_track_unload (t);

		filename = gdl_string_sprintf ("%s/%s.trk", t->dir, chrom->name);

		if (gdl_isfile (filename))
		{
			stream   = gdl_fileopen (filename, "r");
			t->chrom = (t->T->load)(stream);
			gdl_fileclose (filename, stream);
			status = GDL_SUCCESS;
		}
		else
		{
			t->chrom = 0;
		}
		gdl_string_free (filename);
	}
	else
	{
		status = !strcmp(t->T->name, gdl_igdb_cis_annot_snp_table->name) ? GDL_SUCCESS : status; // no loader (=snp_table)
	}

	return status;
}

size_t
gdl_igdb_cis_annot_track_focus (const gdl_igdb_cis_annot_track * t, gdl_igdb_cis_annot_chromosome ** t_focus, const long from, const long to)
{
	if (!t->chrom) return 0;

	size_t i;
	const gdl_igdb_cis_annot_chromosome * c = t->chrom;
	gdl_igdb_cis_annot_entry * e;
	gdl_list * focus = gdl_list_alloc (gdl_list_default);

	if (  !strcmp(t->T->name, gdl_igdb_cis_annot_intron->name)
	   || !strcmp(t->T->name, gdl_igdb_cis_annot_cds_intron->name)
	   || !strcmp(t->T->name, gdl_igdb_cis_annot_notcds_intron->name)
	   || !strcmp(t->T->name, gdl_igdb_cis_annot_first_intron->name)
	   || !strcmp(t->T->name, gdl_igdb_cis_annot_last_intron->name)
	   || !strcmp(t->T->name, gdl_igdb_cis_annot_exon->name)
	   || !strcmp(t->T->name, gdl_igdb_cis_annot_cds_exon->name)
	   || !strcmp(t->T->name, gdl_igdb_cis_annot_notcds_exon->name)
	   || !strcmp(t->T->name, gdl_igdb_cis_annot_first_exon->name)
	   || !strcmp(t->T->name, gdl_igdb_cis_annot_last_exon->name)
	   || !strcmp(t->T->name, gdl_igdb_cis_annot_5_utr->name)
	   || !strcmp(t->T->name, gdl_igdb_cis_annot_3_utr->name)
	   || !strcmp(t->T->name, gdl_igdb_cis_annot_snp_table->name)
	   || !strcmp(t->T->name, gdl_igdb_cis_annot_inside_gene->name)
	   || !strcmp(t->T->name, gdl_igdb_cis_annot_inside_feature->name)
	   || !strcmp(t->T->name, gdl_igdb_cis_annot_splice_site->name))
	{
		return 1;
	}

	gdl_igdb_cis_annot_track_unfocus (t, t_focus);

	for (i = 0; i < c->size; i++)
	{
		e = c->entries[i];
		if (e->start >= from && e->start <= to)
		{
			gdl_list_push_back (focus, e, 0);
		}
	}

	(*t_focus) = gdl_igdb_cis_annot_chromosome_alloc ();
	(*t_focus)->size = gdl_list_size(focus);

	if ((*t_focus)->size)
	{
		(*t_focus)->entries = GDL_MALLOC (gdl_igdb_cis_annot_entry *, (*t_focus)->size);
		gdl_list_itr * focus_itr = gdl_list_iterator_front (focus);
		i=0;
		do
		{
			(*t_focus)->entries[i] = gdl_list_iterator_value (focus_itr);
			i++;
		}
		while (gdl_list_iterator_next (focus_itr));
		gdl_list_iterator_free (focus_itr);
	}

	gdl_list_free (focus);

	return (*t_focus)->size;
}

void
gdl_igdb_cis_annot_track_unfocus (const gdl_igdb_cis_annot_track * t, gdl_igdb_cis_annot_chromosome ** t_focus)
{
	if (*t_focus)
	{
		GDL_FREE ((*t_focus)->entries);
		GDL_FREE ((*t_focus));
		*t_focus=0;
	}
}

double
gdl_igdb_cis_annot_track_snp_value (gdl_igdb_cis_annot_track * t, const gdl_igdb_cis_annot_chromosome * t_focus, const gdl_igdb_chromosome * chrom, const size_t gene_idx, const int unit_idx, const long position, const int pos_bin, const int snp_idx, size_t * is_discrete)
{
	if (t->T->load && !t->chrom)
	{
		return 0;
	}

	if (t_focus && t_focus->size)
	{
		return (t->T->snp_value)(t->T, t_focus, chrom, gene_idx, unit_idx, position, pos_bin, snp_idx, is_discrete);
	}
	else if (!t_focus)
	{
		return (t->T->snp_value)(t->T, t->chrom, chrom, gene_idx, unit_idx, position, pos_bin, snp_idx, is_discrete);
	}
}

long
gdl_igdb_cis_annot_track_psize (gdl_igdb_cis_annot_track * t, const gdl_igdb_cis_annot_chromosome * t_focus, const gdl_igdb_chromosome * chrom, const size_t gene_idx)
{
	if (t->T->load && !t->chrom) return 0;
	if (!t->T->p_size) return 0;

	if (t_focus && t_focus->size)
	{
		return (t->T->p_size)(t->T, t_focus, chrom, gene_idx);
	}
	else if (!t_focus)
	{
		return (t->T->p_size)(t->T, t->chrom, chrom, gene_idx);
	}
}

int
gdl_igdb_cis_annot_track_unload (gdl_igdb_cis_annot_track * t)
{
	gdl_igdb_cis_annot_chromosome_free (t->chrom);
	t->chrom=0;
}
/*==================================================================
 *
 * UTILITIES
 *
 * ================================================================*/
static size_t
_gene_only_check (const size_t gene_only, const long a_start, const long a_end, const long featureStart, const long featureEnd)
{
	if (gene_only == 0)
	{
		return 0;
	}
	if (gene_only == 1 && (a_start > featureEnd || a_end < featureStart))
	{
		return 1;
	}
	else if (gene_only==2 && (a_start >= featureStart && a_end <= featureEnd))
	{
		return 1;
	}
	return 0;
}

static size_t
_bin_check (const size_t bin_only, const size_t * bin_idx, const int snp_bin)
{
	if (snp_bin != -1 && bin_only && bin_idx)
	{
		size_t i;
		for(i = 0 ; i < bin_only;i++)
		{
			if (snp_bin == bin_idx[i])
				break;
		}
		if (i == bin_only)
		{
			return 1;
		}
	}
	return 0;
}

static gdl_boolean
_anchor_check (const gdl_igdb_cis_annot_track_type * T, const gdl_expr_chromosome * chrom, const gdl_expr_feature * feature, const long position)
{
	if (T->nanchor)
	{
		size_t i, as = 0;
		for(i = 0; i < T->nanchor; i++)
		{
			long d0 = (T->anchors[i]->distance_min)(chrom, feature, position);
			as += (d0 >= T->anchor_ranges[i][0] && d0 <= T->anchor_ranges[i][1]);
		}
		if (!as)
		{
			return 1;
		}
	}
	return 0;
}
/*==================================================================
 *
 * GENOME TRACK - INSIDE
 *
 * ================================================================*/
static long
gdl_igdb_cis_annot_genome_track_psize (const void * vtype, const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t gene_idx)
{
	size_t i,ns,ne;
	const gdl_igdb_cis_annot_track_type * T = (gdl_igdb_cis_annot_track_type *) vtype;
	const size_t gene_only = T->gene_only;
	const gdl_expr_feature * feature = chrom->ftr_data->features[gene_idx];
	const long featureStart = gdl_expr_feature_get_min_start (feature, chrom->exp_data);
	const long featureEnd   = gdl_expr_feature_get_max_end (feature, chrom->exp_data);
	long size=0;

	for(i=0;i<annot->size;i++)
	{
		long a_start = annot->entries[i]->start;
		long a_end   = annot->entries[i]->end;
		if (_gene_only_check (gene_only, a_start, a_end, featureStart, featureEnd))
		{
			continue;
		}
		size+=a_end-a_start+1;
		if (gene_only==1)
		{
			if (a_start < featureStart)
				size -= (featureStart-a_start);
			if (a_end > featureEnd)
				size -= (a_end-featureEnd);
		}
		else if (gene_only==2)
		{
			if (a_start > featureStart && a_start <= featureEnd)
				size -= (featureEnd-a_start);
			if (a_end > featureStart && a_end <= featureEnd)
				size -= (a_end-featureStart);
		}
	}

	return size;
}

static double
gdl_igdb_cis_annot_genome_track_inside (const void * vtype, const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t gene_idx, const int unit_idx, const long position, const int pos_bin, const int snp_idx, size_t * is_discrete)
{
	size_t i;
	const gdl_igdb_cis_annot_track_type * T = (gdl_igdb_cis_annot_track_type *) vtype;
	const size_t gene_only                  = T->gene_only;
	const size_t bin_only                   = T->bin_only;
	const size_t * bin_idx                  = T->bin_idx;
	const gdl_expr_feature * feature        = chrom->ftr_data->features[gene_idx];
	const long featureStart                 = gdl_expr_feature_get_min_start (feature, chrom->exp_data);
	const long featureEnd                   = gdl_expr_feature_get_max_end (feature, chrom->exp_data);

	*is_discrete = 1;

	if (_gene_only_check (gene_only, position, position, featureStart, featureEnd))
	{
		return 0.;
	}
	if (_bin_check (bin_only, bin_idx, pos_bin))
	{
		return 0.;
	}
	if (_anchor_check (T, chrom->exp_data, feature, position))
	{
		return 0.;
	}
	for(i = 0;i < annot->size; i++)
	{
		if (position >= annot->entries[i]->start && position <= annot->entries[i]->end)
		{
			return 1.;
		}
	}

	return 0.;
}
/*==================================================================
 *
 * GENOME SCORE - INSIDE
 *
 * ================================================================*/
static double
gdl_igdb_cis_annot_genome_score_inside (const void * vtype, const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t gene_idx, const int unit_idx, const long position, const int pos_bin, const int snp_idx, size_t * is_discrete)
{
	size_t i;
	const gdl_igdb_cis_annot_track_type * T = (gdl_igdb_cis_annot_track_type *) vtype;
	const size_t gene_only                  = T->gene_only;
	const size_t bin_only                   = T->bin_only;
	const size_t * bin_idx                  = T->bin_idx;
	const gdl_expr_feature * feature        = chrom->ftr_data->features[gene_idx];
	const long featureStart                 = gdl_expr_feature_get_min_start (feature, chrom->exp_data);
	const long featureEnd                   = gdl_expr_feature_get_max_end (feature, chrom->exp_data);

	*is_discrete = 1;

	if (_gene_only_check (gene_only, position, position, featureStart, featureEnd))
	{
		return 0;
	}
	if (_bin_check (bin_only, bin_idx, pos_bin))
	{
		return 0;
	}
	if (_anchor_check (T, chrom->exp_data, feature, position))
	{
		return 0;
	}
	for(i = 0;i < annot->size; i++)
	{
		if (position >= annot->entries[i]->start && position <= annot->entries[i]->end)
		{
			return annot->entries[i]->score;
		}
	}

	return 0;
}
/*==================================================================
 *
 * SNP TABLE
 *
 * ================================================================*/
static double
gdl_igdb_cis_annot_snp_table_value (const void * vtype, const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t gene_idx, const int unit_idx, const long position, const int pos_bin, const int snp_idx, size_t * is_discrete)
{
	const gdl_igdb_cis_annot_track_type * T = (gdl_igdb_cis_annot_track_type *) vtype;
	const gdl_snp * snp;
	const gdl_expr_feature * feature = chrom->ftr_data->features[gene_idx];

	*is_discrete = 1;

	if (snp_idx != -1)
	{
		snp = chrom->snp_data->chrom->snps[snp_idx];
		double x = gdl_snp_table_get_score ((gdl_snp_table *)T->extra, chrom->name, feature->name, snp->rs);
		return (gdl_isnan(x) ? 0.0 : x);
	}

	return 0;
}
/*==================================================================
 *
 * 5' UTR
 *
 * ================================================================*/
static double
gdl_igdb_cis_annot_snp_5_utr (const void * vtype, const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t gene_idx, const int unit_idx, const long position, const int pos_bin, const int snp_idx, size_t * is_discrete)
{
	size_t i,n;
	const gdl_igdb_cis_annot_track_type * T = (gdl_igdb_cis_annot_track_type *) vtype;
	const size_t bin_only                   = T->bin_only;
	const size_t * bin_idx                  = T->bin_idx;
	const gdl_expr_feature * feature        = chrom->ftr_data->features[gene_idx];
	const long featureStart                 = gdl_expr_feature_get_min_start (feature, chrom->exp_data);
	const long featureEnd                   = gdl_expr_feature_get_max_end (feature, chrom->exp_data);

	*is_discrete = 1;

	if (_bin_check (bin_only, bin_idx, pos_bin))
	{
		return 0;
	}
	if (_anchor_check (T, chrom->exp_data, feature, position))
	{
		return 0;
	}

	if (feature->T == gdl_expr_feature_transcript)
	{
		gdl_expr_gene * gene    = chrom->exp_data->genes[feature->idx[0]];
		gdl_expr_prbtxset * pts = gene->prbtxsets[feature->idx[1]];

		if (unit_idx != -1)
		{
			gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[unit_idx]];
			return gdl_expr_gene_is_in_5utr (gene, tx, position);
		}
		else
		{
			for(n = i = 0; i < pts->ntx; i++)
			{
				gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[i]];
				n += gdl_expr_gene_is_in_5utr (gene, tx, position);
			}
			*is_discrete = (pts->ntx > 1 && n < pts->ntx) ? 0 : 1;
			return ((double)n) / pts->ntx;
		}
	}
	else
	{
		gdl_expr_gene * gene = chrom->exp_data->genes[feature->idx[0]];
		if (unit_idx != -1)
		{
			gdl_expr_transcript * tx = gene->transcripts[unit_idx];
			return gdl_expr_gene_is_in_5utr (gene, tx, position);
		}
		else
		{
			for(n = i = 0; i < gene->ntx; i++)
			{
				gdl_expr_transcript * tx = gene->transcripts[i];
				n += gdl_expr_gene_is_in_5utr (gene, tx, position);
			}
			*is_discrete = (gene->ntx > 1 && n < gene->ntx) ? 0 : 1;
			return ((double)n) / gene->ntx;
		}
	}

	return 0;
}
/*==================================================================
 *
 * 3' UTR
 *
 * ================================================================*/
static double
gdl_igdb_cis_annot_snp_3_utr (const void * vtype, const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t gene_idx, const int unit_idx, const long position, const int pos_bin, const int snp_idx, size_t * is_discrete)
{
	size_t i,n;
	const gdl_igdb_cis_annot_track_type * T = (gdl_igdb_cis_annot_track_type *) vtype;
	const size_t bin_only                   = T->bin_only;
	const size_t * bin_idx                  = T->bin_idx;
	const gdl_expr_feature * feature        = chrom->ftr_data->features[gene_idx];
	const long featureStart                 = gdl_expr_feature_get_min_start (feature, chrom->exp_data);
	const long featureEnd                   = gdl_expr_feature_get_max_end (feature, chrom->exp_data);

	*is_discrete = 1;

	if (_bin_check (bin_only, bin_idx, pos_bin))
	{
		return 0;
	}
	if (_anchor_check (T, chrom->exp_data, feature, position))
	{
		return 0;
	}

	if (feature->T == gdl_expr_feature_transcript)
	{
		gdl_expr_gene * gene    = chrom->exp_data->genes[feature->idx[0]];
		gdl_expr_prbtxset * pts = gene->prbtxsets[feature->idx[1]];
		if (unit_idx != -1)
		{
			gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[unit_idx]];
			return gdl_expr_gene_is_in_3utr (gene, tx, position);
		}
		else
		{
			for(n = i = 0; i < pts->ntx; i++)
			{
				gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[i]];
				n += gdl_expr_gene_is_in_3utr (gene, tx, position);
			}
			*is_discrete = (pts->ntx > 1 && n < pts->ntx) ? 0 : 1;
			return ((double)n) / pts->ntx;
		}
	}
	else
	{
		gdl_expr_gene * gene = chrom->exp_data->genes[feature->idx[0]];
		if (unit_idx != -1)
		{
			gdl_expr_transcript * tx = gene->transcripts[unit_idx];
			return gdl_expr_gene_is_in_3utr (gene, tx, position);
		}
		else
		{
			for(n = i = 0; i < gene->ntx; i++)
			{
				gdl_expr_transcript * tx = gene->transcripts[i];
				n += gdl_expr_gene_is_in_3utr (gene, tx, position);
			}
			*is_discrete = (gene->ntx > 1 && n < gene->ntx) ? 0 : 1;
			return ((double)n) / gene->ntx;
		}
	}

	return 0;

}
/*==================================================================
 *
 * Inside Feature (means inside the feature)
 *
 * ================================================================*/
static double
gdl_igdb_cis_annot_snp_inside_feature (const void * vtype, const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t gene_idx, const int unit_idx, const long position, const int pos_bin, const int snp_bin, size_t * is_discrete)
{
	size_t i,n;
	const gdl_igdb_cis_annot_track_type * T = (gdl_igdb_cis_annot_track_type *) vtype;
	const size_t bin_only                   = T->bin_only;
	const size_t * bin_idx                  = T->bin_idx;
	const gdl_expr_feature * feature        = chrom->ftr_data->features[gene_idx];
	const long featureStart                 = gdl_expr_feature_get_min_start (feature, chrom->exp_data);
	const long featureEnd                   = gdl_expr_feature_get_max_end (feature, chrom->exp_data);

	*is_discrete = 1;

	if (_bin_check (bin_only, bin_idx, pos_bin))
	{
		return 0;
	}
	if (_anchor_check (T, chrom->exp_data, feature, position))
	{
		return 0;
	}
	if (position >= featureStart && position <= featureEnd)
	{
		return 1;
	}
	return 0;
}
/*==================================================================
 *
 * Inside Gene (means inside transcript)
 *
 * ================================================================*/
static double
gdl_igdb_cis_annot_snp_inside_gene (const void * vtype, const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t gene_idx, const int unit_idx, const long position, const int pos_bin, const int snp_bin, size_t * is_discrete)
{
	size_t i,n;
	const gdl_igdb_cis_annot_track_type * T = (gdl_igdb_cis_annot_track_type *) vtype;
	const size_t bin_only                   = T->bin_only;
	const size_t * bin_idx                  = T->bin_idx;
	const gdl_expr_feature * feature        = chrom->ftr_data->features[gene_idx];
	const long featureStart                 = gdl_expr_feature_get_min_start (feature, chrom->exp_data);
	const long featureEnd                   = gdl_expr_feature_get_max_end (feature, chrom->exp_data);

	*is_discrete = 1;

	if (_bin_check (bin_only, bin_idx, pos_bin))
	{
		return 0;
	}
	if (_anchor_check (T, chrom->exp_data, feature, position))
	{
		return 0;
	}

	if (feature->T == gdl_expr_feature_transcript)
	{
		gdl_expr_gene * gene    = chrom->exp_data->genes[feature->idx[0]];
		gdl_expr_prbtxset * pts = gene->prbtxsets[feature->idx[1]];
		if (unit_idx != -1)
		{
			gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[unit_idx]];
			return gdl_expr_gene_is_within (gene, tx, position);
		}
		else
		{
			for(n = i = 0; i < pts->ntx; i++)
			{
				gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[i]];
				n += gdl_expr_gene_is_within (gene, tx, position);
			}
			*is_discrete = (pts->ntx > 1 && n < pts->ntx) ? 0 : 1;
			return ((double)n) / pts->ntx;
		}
	}
	else
	{
		gdl_expr_gene * gene = chrom->exp_data->genes[feature->idx[0]];
		if (unit_idx != -1)
		{
			gdl_expr_transcript * tx = gene->transcripts[unit_idx];
			return gdl_expr_gene_is_within (gene, tx, position);
		}
		else
		{
			for(n = i = 0; i < gene->ntx; i++)
			{
				gdl_expr_transcript * tx = gene->transcripts[i];
				n += gdl_expr_gene_is_within (gene, tx, position);
			}
			*is_discrete = (gene->ntx > 1 && n < gene->ntx) ? 0 : 1;
			return ((double)n) / gene->ntx;
		}
	}

	return 0;
}
/*==================================================================
 *
 * Intron whatever its kind
 *
 * ================================================================*/
static double
gdl_igdb_cis_annot_snp_intron (const void * vtype,  const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t gene_idx, const int unit_idx, const long position, const int pos_bin, const int snp_idx, size_t * is_discrete)
{
	size_t i,n;
	int j;
	gdl_boolean in_exon = 0;
	const gdl_igdb_cis_annot_track_type * T = (gdl_igdb_cis_annot_track_type *) vtype;
	const size_t bin_only                   = T->bin_only;
	const size_t * bin_idx                  = T->bin_idx;
	const gdl_expr_feature * feature        = chrom->ftr_data->features[gene_idx];
	const long featureStart                 = gdl_expr_feature_get_min_start (feature, chrom->exp_data);
	const long featureEnd                   = gdl_expr_feature_get_max_end (feature, chrom->exp_data);

	*is_discrete = 1;

	if (_bin_check (bin_only, bin_idx, pos_bin))
	{
		return 0;
	}
	if (_anchor_check (T, chrom->exp_data, feature, position))
	{
		return 0;
	}

	if (feature->T == gdl_expr_feature_transcript)
	{
		gdl_expr_gene * gene    = chrom->exp_data->genes[feature->idx[0]];
		gdl_expr_prbtxset * pts = gene->prbtxsets[feature->idx[1]];
		if (unit_idx != -1)
		{
			gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[unit_idx]];
			return gdl_expr_gene_is_in_intron (gene, tx, position);
		}
		else
		{
			for(n = i = 0; i < pts->ntx; i++)
			{
				gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[i]];
				n += gdl_expr_gene_is_in_intron (gene, tx, position);
			}
			*is_discrete = (pts->ntx > 1 && n < pts->ntx) ? 0 : 1;
			return ((double)n) / pts->ntx;
		}
	}
	else
	{
		gdl_expr_gene * gene = chrom->exp_data->genes[feature->idx[0]];
		if (unit_idx != -1)
		{
			gdl_expr_transcript * tx = gene->transcripts[unit_idx];
			return gdl_expr_gene_is_in_intron (gene, tx, position);
		}
		else
		{
			for(n = i = 0; i < gene->ntx; i++)
			{
				gdl_expr_transcript * tx = gene->transcripts[i];
				n += gdl_expr_gene_is_in_intron (gene, tx, position);
			}
			*is_discrete = (gene->ntx > 1 && n < gene->ntx) ? 0 : 1;
			return ((double)n) / gene->ntx;
		}
	}

	return 0;
}

static long
gdl_igdb_cis_annot_intron_psize (const void * vtype, const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t gene_idx)
{
	size_t i, j;
	gdl_boolean in_exon = 0;
	const gdl_igdb_cis_annot_track_type * T = (gdl_igdb_cis_annot_track_type *) vtype;
	const size_t bin_only                   = T->bin_only;
	const size_t * bin_idx                  = T->bin_idx;
	const gdl_expr_feature * feature        = chrom->ftr_data->features[gene_idx];
	long ex_tot, tot = 0;

	if (feature->T == gdl_expr_feature_transcript)
	{
		gdl_expr_gene * gene    = chrom->exp_data->genes[feature->idx[0]];
		gdl_expr_prbtxset * pts = gene->prbtxsets[feature->idx[1]];
		for(i = 0; i < pts->ntx; i++)
		{
			gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[i]];
			ex_tot = 0;
			for (j = 0; j < tx->nexon; j++)
			{
				ex_tot += gene->exons[tx->exon_idx[j]]->end-gene->exons[tx->exon_idx[j]]->start+1;
			}
			tot += (tx->txEnd - tx->txStart + 1) - ex_tot;
		}
		tot /= pts->ntx;
	}
	else
	{
		gdl_expr_gene * gene = chrom->exp_data->genes[feature->idx[0]];
		for(i = 0; i < gene->ntx; i++)
		{
			gdl_expr_transcript * tx = gene->transcripts[i];
			ex_tot = 0;
			for (j = 0; j < tx->nexon; j++)
			{
				ex_tot += gene->exons[tx->exon_idx[j]]->end-gene->exons[tx->exon_idx[j]]->start+1;
			}
			tot += (tx->txEnd - tx->txStart + 1) - ex_tot;
		}
		tot /= gene->ntx;
	}

	return tot;
}
/*==================================================================
 *
 * Part of intron located within the CDS
 *
 * ================================================================*/
static double
gdl_igdb_cis_annot_snp_cds_intron (const void * vtype,  const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t gene_idx, const int unit_idx, const long position, const int pos_bin, const int snp_idx, size_t * is_discrete)
{
	size_t i,n;
	int j;
	gdl_boolean in_exon = 0;
	const gdl_igdb_cis_annot_track_type * T = (gdl_igdb_cis_annot_track_type *) vtype;
	const size_t bin_only                   = T->bin_only;
	const size_t * bin_idx                  = T->bin_idx;
	const gdl_expr_feature * feature        = chrom->ftr_data->features[gene_idx];
	const long featureStart                 = gdl_expr_feature_get_min_start (feature, chrom->exp_data);
	const long featureEnd                   = gdl_expr_feature_get_max_end (feature, chrom->exp_data);

	*is_discrete = 1;

	if (_bin_check (bin_only, bin_idx, pos_bin))
	{
		return 0;
	}
	if (_anchor_check (T, chrom->exp_data, feature, position))
	{
		return 0;
	}

	if (feature->T == gdl_expr_feature_transcript)
	{
		gdl_expr_gene * gene    = chrom->exp_data->genes[feature->idx[0]];
		gdl_expr_prbtxset * pts = gene->prbtxsets[feature->idx[1]];
		if (unit_idx != -1)
		{
			gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[unit_idx]];
			return gdl_expr_gene_is_in_cds_intron (gene, tx, position);
		}
		else
		{
			for(n = i = 0; i < pts->ntx; i++)
			{
				gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[i]];
				n += gdl_expr_gene_is_in_cds_intron (gene, tx, position);
			}
			*is_discrete = (pts->ntx > 1 && n < pts->ntx) ? 0 : 1;
			return ((double)n) / pts->ntx;
		}
	}
	else
	{
		gdl_expr_gene * gene = chrom->exp_data->genes[feature->idx[0]];
		if (unit_idx != -1)
		{
			gdl_expr_transcript * tx = gene->transcripts[unit_idx];
			return gdl_expr_gene_is_in_cds_intron (gene, tx, position);
		}
		else
		{
			for(n = i = 0; i < gene->ntx; i++)
			{
				gdl_expr_transcript * tx = gene->transcripts[i];
				n += gdl_expr_gene_is_in_cds_intron (gene, tx, position);
			}
			*is_discrete = (gene->ntx > 1 && n < gene->ntx) ? 0 : 1;
			return ((double)n) / gene->ntx;
		}
	}

	return 0;
}

static long
gdl_igdb_cis_annot_cds_intron_psize (const void * vtype, const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t gene_idx)
{
	size_t i, j;
	gdl_boolean in_exon = 0;
	const gdl_igdb_cis_annot_track_type * T = (gdl_igdb_cis_annot_track_type *) vtype;
	const size_t bin_only                   = T->bin_only;
	const size_t * bin_idx                  = T->bin_idx;
	const gdl_expr_feature * feature        = chrom->ftr_data->features[gene_idx];
	long ex_tot, tot = 0;

	if (feature->T == gdl_expr_feature_transcript)
	{
		gdl_expr_gene * gene    = chrom->exp_data->genes[feature->idx[0]];
		gdl_expr_prbtxset * pts = gene->prbtxsets[feature->idx[1]];
		for(i = 0; i < pts->ntx; i++)
		{
			gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[i]];
			ex_tot = 0;
			for (j = 0; j < tx->nexon; j++)
			{
				long ex_start = gene->exons[tx->exon_idx[j]]->start;
				long ex_end   = gene->exons[tx->exon_idx[j]]->end;
				if (ex_end < tx->cdsStart)
				{
					continue;
				}
				if (ex_start > tx->cdsEnd)
				{
					continue;
				}
				ex_start = GDL_MAX (ex_start, tx->cdsStart);
				ex_end   = GDL_MIN (ex_end, tx->cdsEnd);
				ex_tot  += ex_end - ex_start + 1;
			}
			tot += (tx->cdsEnd - tx->cdsStart + 1) - ex_tot;
		}
		tot /= pts->ntx;
	}
	else
	{
		gdl_expr_gene * gene = chrom->exp_data->genes[feature->idx[0]];
		for(i = 0; i < gene->ntx; i++)
		{
			gdl_expr_transcript * tx = gene->transcripts[i];
			ex_tot = 0;
			for (j = 0; j < tx->nexon; j++)
			{
				long ex_start = gene->exons[tx->exon_idx[j]]->start;
				long ex_end   = gene->exons[tx->exon_idx[j]]->end;
				if (ex_end < tx->cdsStart)
				{
					continue;
				}
				if (ex_start > tx->cdsEnd)
				{
					continue;
				}
				ex_start = GDL_MAX (ex_start, tx->cdsStart);
				ex_end   = GDL_MIN (ex_end, tx->cdsEnd);
				ex_tot  += ex_end - ex_start + 1;
			}
			tot += (tx->cdsEnd - tx->cdsStart + 1) - ex_tot;
		}
		tot /= gene->ntx;
	}

	return tot;
}
/*==================================================================
 *
 * Part of intron located outside the CDS (but not the first or the last)
 *
 * ================================================================*/
static double
gdl_igdb_cis_annot_snp_notcds_intron (const void * vtype, const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t gene_idx, const int unit_idx, const long position, const int pos_bin, const int snp_idx, size_t * is_discrete)
{
	size_t i,n;
	int j;
	gdl_boolean in_exon = 0;
	const gdl_igdb_cis_annot_track_type * T = (gdl_igdb_cis_annot_track_type *) vtype;
	const size_t bin_only                   = T->bin_only;
	const size_t * bin_idx                  = T->bin_idx;
	const gdl_expr_feature * feature        = chrom->ftr_data->features[gene_idx];
	const long featureStart                 = gdl_expr_feature_get_min_start (feature, chrom->exp_data);
	const long featureEnd                   = gdl_expr_feature_get_max_end (feature, chrom->exp_data);
	const unsigned char featureStrand       = (feature->T->get_strand)(chrom->exp_data, feature->idx);

	*is_discrete = 1;

	if (_bin_check (bin_only, bin_idx, pos_bin))
	{
		return 0;
	}
	if (_anchor_check (T, chrom->exp_data, feature, position))
	{
		return 0;
	}

	if (feature->T == gdl_expr_feature_transcript)
	{
		gdl_expr_gene * gene    = chrom->exp_data->genes[feature->idx[0]];
		gdl_expr_prbtxset * pts = gene->prbtxsets[feature->idx[1]];
		if (unit_idx != -1)
		{
			gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[unit_idx]];
			return gdl_expr_gene_is_in_non_cds_intron (gene, tx, position);
		}
		else
		{
			for(n = i = 0; i < pts->ntx; i++)
			{
				gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[i]];
				n += gdl_expr_gene_is_in_non_cds_intron (gene, tx, position);
			}
			*is_discrete = (pts->ntx > 1 && n < pts->ntx) ? 0 : 1;
			return ((double)n) / pts->ntx;
		}
	}
	else
	{
		gdl_expr_gene * gene = chrom->exp_data->genes[feature->idx[0]];
		if (unit_idx != -1)
		{
			gdl_expr_transcript * tx = gene->transcripts[unit_idx];
			return gdl_expr_gene_is_in_non_cds_intron (gene, tx, position);
		}
		else
		{
			for(n = i = 0; i < gene->ntx; i++)
			{
				gdl_expr_transcript * tx = gene->transcripts[i];
				n += gdl_expr_gene_is_in_non_cds_intron (gene, tx, position);
			}
			*is_discrete = (gene->ntx > 1 && n < gene->ntx) ? 0 : 1;
			return ((double)n) / gene->ntx;
		}
	}

	return 0;
}
/*==================================================================
 *
 * First intron
 *
 * ================================================================*/
static double
gdl_igdb_cis_annot_snp_first_intron (const void * vtype, const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t gene_idx, const int unit_idx, const long position, const int pos_bin, const int snp_idx, size_t * is_discrete)
{
	size_t i,n;
	int j;
	gdl_boolean in_exon = 0;
	const gdl_igdb_cis_annot_track_type * T = (gdl_igdb_cis_annot_track_type *) vtype;
	const size_t bin_only                   = T->bin_only;
	const size_t * bin_idx                  = T->bin_idx;
	const gdl_expr_feature * feature        = chrom->ftr_data->features[gene_idx];
	const long featureStart                 = gdl_expr_feature_get_min_start (feature, chrom->exp_data);
	const long featureEnd                   = gdl_expr_feature_get_max_end (feature, chrom->exp_data);
	const unsigned char featureStrand       = (feature->T->get_strand)(chrom->exp_data, feature->idx);

	*is_discrete = 1;

	if (_bin_check (bin_only, bin_idx, pos_bin))
	{
		return 0;
	}
	if (_anchor_check (T, chrom->exp_data, feature, position))
	{
		return 0;
	}

	if (feature->T == gdl_expr_feature_transcript)
	{
		gdl_expr_gene * gene    = chrom->exp_data->genes[feature->idx[0]];
		gdl_expr_prbtxset * pts = gene->prbtxsets[feature->idx[1]];
		if (unit_idx != -1)
		{
			gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[unit_idx]];
			return gdl_expr_gene_is_in_first_intron (gene, tx, position);
		}
		else
		{
			for(n = i = 0; i < pts->ntx; i++)
			{
				gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[i]];
				n += gdl_expr_gene_is_in_first_intron (gene, tx, position);
			}
			*is_discrete = (pts->ntx > 1 && n < pts->ntx) ? 0 : 1;
			return ((double)n) / pts->ntx;
		}
	}
	else
	{
		gdl_expr_gene * gene = chrom->exp_data->genes[feature->idx[0]];
		if (unit_idx != -1)
		{
			gdl_expr_transcript * tx = gene->transcripts[unit_idx];
			return gdl_expr_gene_is_in_first_intron (gene, tx, position);
		}
		else
		{
			for(n = i = 0; i < gene->ntx; i++)
			{
				gdl_expr_transcript * tx = gene->transcripts[i];
				n += gdl_expr_gene_is_in_first_intron (gene, tx, position);
			}
			*is_discrete = (gene->ntx > 1 && n < gene->ntx) ? 0 : 1;
			return ((double)n) / gene->ntx;
		}
	}

	return 0;
}

//static long
//gdl_igdb_cis_annot_first_intron_psize (const void * vtype, const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t gene_idx)
//{
//	const gdl_eqtl_gene * gene = chrom->genes[gene_idx];
//	const nexon                = gene->exonCount;
//	long start=0, end=0;
//	size_t i;
//
//	if (nexon == 1)
//		return 0;
//
//	if (gene->strand == '+')
//	{
//		start = GDL_MIN (gene->cdsStart, gene->exonEnds[0]);
//		end   = GDL_MIN (gene->cdsStart, gene->exonStarts[1]);
//	}
//	else
//	{
//		start = GDL_MAX (gene->cdsEnd, gene->exonEnds[nexon-2]);
//		end   = GDL_MAX (gene->cdsEnd, gene->exonStarts[nexon-1]);
//	}
//
//	return end-start;
//}
/*==================================================================
 *
 * Last intron
 *
 * ================================================================*/
static double
gdl_igdb_cis_annot_snp_last_intron (const void * vtype, const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t gene_idx, const int unit_idx, const long position, const int pos_bin, const int snp_idx, size_t * is_discrete)
{
	size_t i,n;
	int j;
	gdl_boolean in_exon = 0;
	const gdl_igdb_cis_annot_track_type * T = (gdl_igdb_cis_annot_track_type *) vtype;
	const size_t bin_only                   = T->bin_only;
	const size_t * bin_idx                  = T->bin_idx;
	const gdl_expr_feature * feature        = chrom->ftr_data->features[gene_idx];
	const long featureStart                 = gdl_expr_feature_get_min_start (feature, chrom->exp_data);
	const long featureEnd                   = gdl_expr_feature_get_max_end (feature, chrom->exp_data);
	const unsigned char featureStrand       = (feature->T->get_strand)(chrom->exp_data, feature->idx);

	*is_discrete = 1;

	if (_bin_check (bin_only, bin_idx, pos_bin))
	{
		return 0;
	}
	if (_anchor_check (T, chrom->exp_data, feature, position))
	{
		return 0;
	}

	if (feature->T == gdl_expr_feature_transcript)
	{
		gdl_expr_gene * gene    = chrom->exp_data->genes[feature->idx[0]];
		gdl_expr_prbtxset * pts = gene->prbtxsets[feature->idx[1]];
		if (unit_idx != -1)
		{
			gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[unit_idx]];
			return gdl_expr_gene_is_in_last_intron (gene, tx, position);
		}
		else
		{
			for(n = i = 0; i < pts->ntx; i++)
			{
				gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[i]];
				n += gdl_expr_gene_is_in_last_intron (gene, tx, position);
			}
			*is_discrete = (pts->ntx > 1 && n < pts->ntx) ? 0 : 1;
			return ((double)n) / pts->ntx;
		}
	}
	else
	{
		gdl_expr_gene * gene = chrom->exp_data->genes[feature->idx[0]];
		if (unit_idx != -1)
		{
			gdl_expr_transcript * tx = gene->transcripts[unit_idx];
			return gdl_expr_gene_is_in_last_intron (gene, tx, position);
		}
		else
		{
			for(n = i = 0; i < gene->ntx; i++)
			{
				gdl_expr_transcript * tx = gene->transcripts[i];
				n += gdl_expr_gene_is_in_last_intron (gene, tx, position);
			}
			*is_discrete = (gene->ntx > 1 && n < gene->ntx) ? 0 : 1;
			return ((double)n) / gene->ntx;
		}
	}

	return 0;
}

//static long
//gdl_igdb_cis_annot_last_intron_psize (const void * vtype, const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t gene_idx)
//{
//	const gdl_eqtl_gene * gene = chrom->genes[gene_idx];
//	const nexon                = gene->exonCount;
//	long start=0, end=0;
//	size_t i;
//
//	if (nexon <= 2)
//		return 0;
//
//	if (gene->strand == '+')
//	{
//		start = GDL_MAX (gene->cdsEnd, gene->exonEnds[nexon-2]);
//		end   = GDL_MAX (gene->cdsEnd, gene->exonStarts[nexon-1]);
//	}
//	else
//	{
//		start = GDL_MIN (gene->cdsStart, gene->exonEnds[0]);
//		end   = GDL_MIN (gene->cdsStart, gene->exonStarts[1]);
//	}
//
//	return end-start;
//}

//static long
//gdl_igdb_cis_annot_notcds_intron_psize (const void * vtype, const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t gene_idx)
//{
//	size_t i;
//	const gdl_eqtl_gene * gene = chrom->genes[gene_idx];
//	long intron_size = 0;
//	long other_intron = gdl_igdb_cis_annot_cds_intron_psize (vtype, annot, chrom, gene_idx);
//   other_intron += gdl_igdb_cis_annot_first_intron_psize (vtype, annot, chrom, gene_idx);
//   other_intron += gdl_igdb_cis_annot_last_intron_psize (vtype, annot, chrom, gene_idx);
//
//   for(i = 0; i < gene->exonCount-1; i++)
//   {
//   	intron_size += gene->exonStarts[i+1]-gene->exonEnds[i];
//   }
//
//   return intron_size-other_intron;
//}
/*==================================================================
 *
 * Exon
 *
 * ================================================================*/
static double
gdl_igdb_cis_annot_snp_exon (const void * vtype, const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t gene_idx, const int unit_idx, const long position, const int pos_bin, const int snp_idx, size_t * is_discrete)
{
	size_t i,n;
	int j;
	gdl_boolean in_exon = 0;
	const gdl_igdb_cis_annot_track_type * T = (gdl_igdb_cis_annot_track_type *) vtype;
	const size_t bin_only                   = T->bin_only;
	const size_t * bin_idx                  = T->bin_idx;
	const gdl_expr_feature * feature        = chrom->ftr_data->features[gene_idx];
	const long featureStart                 = gdl_expr_feature_get_min_start (feature, chrom->exp_data);
	const long featureEnd                   = gdl_expr_feature_get_max_end (feature, chrom->exp_data);
	const unsigned char featureStrand       = (feature->T->get_strand)(chrom->exp_data, feature->idx);

	*is_discrete = 1;

	if (_bin_check (bin_only, bin_idx, pos_bin))
	{
		return 0;
	}
	if (_anchor_check (T, chrom->exp_data, feature, position))
	{
		return 0;
	}

	if (feature->T == gdl_expr_feature_transcript)
	{
		gdl_expr_gene * gene    = chrom->exp_data->genes[feature->idx[0]];
		gdl_expr_prbtxset * pts = gene->prbtxsets[feature->idx[1]];
		if (unit_idx != -1)
		{
			gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[unit_idx]];
			return gdl_expr_gene_is_in_exon (gene, tx, position);
		}
		else
		{
			for(n = i = 0; i < pts->ntx; i++)
			{
				gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[i]];
				n += gdl_expr_gene_is_in_exon (gene, tx, position);
			}
			*is_discrete = (pts->ntx > 1 && n < pts->ntx) ? 0 : 1;
			return ((double)n) / pts->ntx;
		}
	}
	else
	{
		gdl_expr_gene * gene = chrom->exp_data->genes[feature->idx[0]];
		if (unit_idx != -1)
		{
			gdl_expr_transcript * tx = gene->transcripts[unit_idx];
			return gdl_expr_gene_is_in_exon (gene, tx, position);
		}
		else
		{
			for(n = i = 0; i < gene->ntx; i++)
			{
				gdl_expr_transcript * tx = gene->transcripts[i];
				n += gdl_expr_gene_is_in_exon (gene, tx, position);
			}
			*is_discrete = (gene->ntx > 1 && n < gene->ntx) ? 0 : 1;
			return ((double)n) / gene->ntx;
		}
	}

	return 0;
}
/*==================================================================
 *
 * CDS exon = part of exons that is coding
 *
 * ================================================================*/
static double
gdl_igdb_cis_annot_snp_cds_exon (const void * vtype, const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t gene_idx, const int unit_idx, const long position, const int pos_bin, const int snp_idx, size_t * is_discrete)
{
	size_t i,n;
	int j;
	gdl_boolean in_exon = 0;
	const gdl_igdb_cis_annot_track_type * T = (gdl_igdb_cis_annot_track_type *) vtype;
	const size_t bin_only                   = T->bin_only;
	const size_t * bin_idx                  = T->bin_idx;
	const gdl_expr_feature * feature        = chrom->ftr_data->features[gene_idx];
	const long featureStart                 = gdl_expr_feature_get_min_start (feature, chrom->exp_data);
	const long featureEnd                   = gdl_expr_feature_get_max_end (feature, chrom->exp_data);
	const unsigned char featureStrand       = (feature->T->get_strand)(chrom->exp_data, feature->idx);

	*is_discrete = 1;

	if (_bin_check (bin_only, bin_idx, pos_bin))
	{
		return 0;
	}
	if (_anchor_check (T, chrom->exp_data, feature, position))
	{
		return 0;
	}

	if (feature->T == gdl_expr_feature_transcript)
	{
		gdl_expr_gene * gene    = chrom->exp_data->genes[feature->idx[0]];
		gdl_expr_prbtxset * pts = gene->prbtxsets[feature->idx[1]];
		if (unit_idx != -1)
		{
			gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[unit_idx]];
			return gdl_expr_gene_is_in_cds_exon (gene, tx, position);
		}
		else
		{
			for(n = i = 0; i < pts->ntx; i++)
			{
				gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[i]];
				n += gdl_expr_gene_is_in_cds_exon (gene, tx, position);
			}
			*is_discrete = (pts->ntx > 1 && n < pts->ntx) ? 0 : 1;
			return ((double)n) / pts->ntx;
		}
	}
	else
	{
		gdl_expr_gene * gene = chrom->exp_data->genes[feature->idx[0]];
		if (unit_idx != -1)
		{
			gdl_expr_transcript * tx = gene->transcripts[unit_idx];
			return gdl_expr_gene_is_in_cds_exon (gene, tx, position);
		}
		else
		{
			for(n = i = 0; i < gene->ntx; i++)
			{
				gdl_expr_transcript * tx = gene->transcripts[i];
				n += gdl_expr_gene_is_in_cds_exon (gene, tx, position);
			}
			*is_discrete = (gene->ntx > 1 && n < gene->ntx) ? 0 : 1;
			return ((double)n) / gene->ntx;
		}
	}

	return 0;
}

//static long
//gdl_igdb_cis_annot_cds_exon_psize (const void * vtype, const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t gene_idx)
//{
//	const gdl_eqtl_gene * gene = chrom->genes[gene_idx];
//	const nexon                = gene->exonCount;
//	long  cds_exon             = 0;
//	size_t i;
//
//	for(i = 0; i < nexon; i++)
//	{
//		if (gene->exonEnds[i] >= gene->cdsStart && gene->exonStarts[i] <= gene->cdsEnd)
//		{
//			 long start = GDL_MAX (gene->cdsStart, gene->exonStarts[i]);
//			 long end   = GDL_MIN (gene->cdsEnd, gene->exonEnds[i]);
//			 cds_exon  += end-start;
//		}
//	}
//
//	return cds_exon;
//}
/*==================================================================
 *
 * Non CDS exon = part of exons that are outisde the coding region
 * but which are neither the first or the last exon
 *
 * ================================================================*/
static double
gdl_igdb_cis_annot_snp_notcds_exon (const void * vtype, const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t gene_idx, const int unit_idx, const long position, const int pos_bin, const int snp_idx, size_t * is_discrete)
{
	size_t i,n;
	int j;
	gdl_boolean in_exon = 0;
	const gdl_igdb_cis_annot_track_type * T = (gdl_igdb_cis_annot_track_type *) vtype;
	const size_t bin_only                   = T->bin_only;
	const size_t * bin_idx                  = T->bin_idx;
	const gdl_expr_feature * feature        = chrom->ftr_data->features[gene_idx];
	const long featureStart                 = gdl_expr_feature_get_min_start (feature, chrom->exp_data);
	const long featureEnd                   = gdl_expr_feature_get_max_end (feature, chrom->exp_data);
	const unsigned char featureStrand       = (feature->T->get_strand)(chrom->exp_data, feature->idx);

	*is_discrete = 1;

	if (_bin_check (bin_only, bin_idx, pos_bin))
	{
		return 0;
	}
	if (_anchor_check (T, chrom->exp_data, feature, position))
	{
		return 0;
	}

	if (feature->T == gdl_expr_feature_transcript)
	{
		gdl_expr_gene * gene    = chrom->exp_data->genes[feature->idx[0]];
		gdl_expr_prbtxset * pts = gene->prbtxsets[feature->idx[1]];
		if (unit_idx != -1)
		{
			gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[unit_idx]];
			return gdl_expr_gene_is_in_non_cds_exon (gene, tx, position);
		}
		else
		{
			for(n = i = 0; i < pts->ntx; i++)
			{
				gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[i]];
				n += gdl_expr_gene_is_in_non_cds_exon (gene, tx, position);
			}
			*is_discrete = (pts->ntx > 1 && n < pts->ntx) ? 0 : 1;
			return ((double)n) / pts->ntx;
		}
	}
	else
	{
		gdl_expr_gene * gene = chrom->exp_data->genes[feature->idx[0]];
		if (unit_idx != -1)
		{
			gdl_expr_transcript * tx = gene->transcripts[unit_idx];
			return gdl_expr_gene_is_in_non_cds_exon (gene, tx, position);
		}
		else
		{
			for(n = i = 0; i < gene->ntx; i++)
			{
				gdl_expr_transcript * tx = gene->transcripts[i];
				n += gdl_expr_gene_is_in_non_cds_exon (gene, tx, position);
			}
			*is_discrete = (gene->ntx > 1 && n < gene->ntx) ? 0 : 1;
			return ((double)n) / gene->ntx;
		}
	}

	return 0;
}
/*==================================================================
 *
 * First exon
 *
 * ================================================================*/
static double
gdl_igdb_cis_annot_snp_first_exon (const void * vtype, const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t gene_idx, const int unit_idx, const long position, const int pos_bin, const int snp_idx, size_t * is_discrete)
{
	size_t i,n;
	int j;
	gdl_boolean in_exon = 0;
	const gdl_igdb_cis_annot_track_type * T = (gdl_igdb_cis_annot_track_type *) vtype;
	const size_t bin_only                   = T->bin_only;
	const size_t * bin_idx                  = T->bin_idx;
	const gdl_expr_feature * feature        = chrom->ftr_data->features[gene_idx];
	const long featureStart                 = gdl_expr_feature_get_min_start (feature, chrom->exp_data);
	const long featureEnd                   = gdl_expr_feature_get_max_end (feature, chrom->exp_data);
	const unsigned char featureStrand       = (feature->T->get_strand)(chrom->exp_data, feature->idx);

	*is_discrete = 1;

	if (_bin_check (bin_only, bin_idx, pos_bin))
	{
		return 0;
	}
	if (_anchor_check (T, chrom->exp_data, feature, position))
	{
		return 0;
	}

	if (feature->T == gdl_expr_feature_transcript)
	{
		gdl_expr_gene * gene    = chrom->exp_data->genes[feature->idx[0]];
		gdl_expr_prbtxset * pts = gene->prbtxsets[feature->idx[1]];
		if (unit_idx != -1)
		{
			gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[unit_idx]];
			return gdl_expr_gene_is_in_first_exon (gene, tx, position);
		}
		else
		{
			for(n = i = 0; i < pts->ntx; i++)
			{
				gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[i]];
				n += gdl_expr_gene_is_in_first_exon (gene, tx, position);
			}
			*is_discrete = (pts->ntx > 1 && n < pts->ntx) ? 0 : 1;
			return ((double)n) / pts->ntx;
		}
	}
	else
	{
		gdl_expr_gene * gene = chrom->exp_data->genes[feature->idx[0]];
		if (unit_idx != -1)
		{
			gdl_expr_transcript * tx = gene->transcripts[unit_idx];
			return gdl_expr_gene_is_in_first_exon (gene, tx, position);
		}
		else
		{
			for(n = i = 0; i < gene->ntx; i++)
			{
				gdl_expr_transcript * tx = gene->transcripts[i];
				n += gdl_expr_gene_is_in_first_exon (gene, tx, position);
			}
			*is_discrete = (gene->ntx > 1 && n < gene->ntx) ? 0 : 1;
			return ((double)n) / gene->ntx;
		}
	}

	return 0;
}

//static long
//gdl_igdb_cis_annot_first_exon_psize (const void * vtype, const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t gene_idx)
//{
//	const gdl_eqtl_gene * gene = chrom->genes[gene_idx];
//	const nexon                = gene->exonCount;
//	long start, end;
//
//	if (nexon == 1)
//		return 0;
//
//	if (gene->strand == '+')
//	{
//		start = gene->exonStarts[0];
//		end   = GDL_MIN (gene->cdsStart, gene->exonEnds[0]);
//	}
//	else
//	{
//		start = GDL_MAX (gene->exonStarts[nexon-1], gene->cdsEnd);
//		end   = gene->exonEnds[nexon-1];
//	}
//
//	if (end > start)
//	{
//		return end-start;
//	}
//	else
//		return 0;
//}
/*==================================================================
 *
 * Last exon
 *
 * ================================================================*/
static double
gdl_igdb_cis_annot_snp_last_exon (const void * vtype, const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t gene_idx, const int unit_idx, const long position, const int pos_bin, const int snp_idx, size_t * is_discrete)
{
	size_t i,n;
	int j;
	gdl_boolean in_exon = 0;
	const gdl_igdb_cis_annot_track_type * T = (gdl_igdb_cis_annot_track_type *) vtype;
	const size_t bin_only                   = T->bin_only;
	const size_t * bin_idx                  = T->bin_idx;
	const gdl_expr_feature * feature        = chrom->ftr_data->features[gene_idx];
	const long featureStart                 = gdl_expr_feature_get_min_start (feature, chrom->exp_data);
	const long featureEnd                   = gdl_expr_feature_get_max_end (feature, chrom->exp_data);
	const unsigned char featureStrand       = (feature->T->get_strand)(chrom->exp_data, feature->idx);

	*is_discrete = 1;

	if (_bin_check (bin_only, bin_idx, pos_bin))
	{
		return 0;
	}
	if (_anchor_check (T, chrom->exp_data, feature, position))
	{
		return 0;
	}

	if (feature->T == gdl_expr_feature_transcript)
	{
		gdl_expr_gene * gene    = chrom->exp_data->genes[feature->idx[0]];
		gdl_expr_prbtxset * pts = gene->prbtxsets[feature->idx[1]];
		if (unit_idx != -1)
		{
			gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[unit_idx]];
			return gdl_expr_gene_is_in_last_exon (gene, tx, position);
		}
		else
		{
			for(n = i = 0; i < pts->ntx; i++)
			{
				gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[i]];
				n += gdl_expr_gene_is_in_last_exon (gene, tx, position);
			}
			*is_discrete = (pts->ntx > 1 && n < pts->ntx) ? 0 : 1;
			return ((double)n) / pts->ntx;
		}
	}
	else
	{
		gdl_expr_gene * gene = chrom->exp_data->genes[feature->idx[0]];
		if (unit_idx != -1)
		{
			gdl_expr_transcript * tx = gene->transcripts[unit_idx];
			return gdl_expr_gene_is_in_last_exon (gene, tx, position);
		}
		else
		{
			for(n = i = 0; i < gene->ntx; i++)
			{
				gdl_expr_transcript * tx = gene->transcripts[i];
				n += gdl_expr_gene_is_in_last_exon (gene, tx, position);
			}
			*is_discrete = (gene->ntx > 1 && n < gene->ntx) ? 0 : 1;
			return ((double)n) / gene->ntx;
		}
	}

	return 0;
}
/*=====================================================================================
 *
 * Splice sites
 *
 * =====================================================================================
 */
static double
gdl_igdb_cis_annot_snp_splice_site (const void * vtype,  const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t gene_idx, const int unit_idx, const long position, const int pos_bin, const int snp_idx, size_t * is_discrete)
{
	size_t i,n;
	int j;
	gdl_boolean in_exon = 0;
	const gdl_igdb_cis_annot_track_type * T = (gdl_igdb_cis_annot_track_type *) vtype;
	const size_t bin_only                   = T->bin_only;
	const size_t * bin_idx                  = T->bin_idx;
	const gdl_expr_feature * feature        = chrom->ftr_data->features[gene_idx];
	const long featureStart                 = gdl_expr_feature_get_min_start (feature, chrom->exp_data);
	const long featureEnd                   = gdl_expr_feature_get_max_end (feature, chrom->exp_data);

	*is_discrete = 1;

	if (_bin_check (bin_only, bin_idx, pos_bin))
	{
		return 0;
	}
	if (_anchor_check (T, chrom->exp_data, feature, position))
	{
		return 0;
	}

	gdl_expr_gene * gene    = chrom->exp_data->genes[feature->idx[0]];

	if (feature->T == gdl_expr_feature_transcript)
	{
		gdl_expr_prbtxset * pts = gene->prbtxsets[feature->idx[1]];
		if (unit_idx != -1)
		{
			gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[unit_idx]];
			return gdl_expr_gene_is_in_splice_site (gene, tx, position);
		}
		for(n = i = 0; i < pts->ntx; i++)
		{
			gdl_expr_transcript * tx = gene->transcripts[pts->tx_idx[i]];
			n += gdl_expr_gene_is_in_splice_site (gene, tx, position);
		}
		*is_discrete = (pts->ntx > 1 && n < pts->ntx) ? 0 : 1;
		return ((double)n) / pts->ntx;
	}
	else
	{
		if (unit_idx != -1)
		{
			gdl_expr_transcript * tx = gene->transcripts[unit_idx];
			return gdl_expr_gene_is_in_splice_site (gene, tx, position);
		}
		for(n = i = 0; i < gene->ntx; i++)
		{
			gdl_expr_transcript * tx = gene->transcripts[i];
			n += gdl_expr_gene_is_in_splice_site (gene, tx, position);
		}
		*is_discrete = (gene->ntx > 1 && n < gene->ntx) ? 0 : 1;
		return ((double)n) / gene->ntx;
	}

	return 0;
}

//
//
//static long
//gdl_igdb_cis_annot_last_exon_psize (const void * vtype, const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t gene_idx)
//{
//	const gdl_eqtl_gene * gene = chrom->genes[gene_idx];
//	const nexon                = gene->exonCount;
//	long start, end;
//
//	if (nexon == 1)
//		return 0;
//
//	if (gene->strand == '-')
//	{
//		start = gene->exonStarts[0];
//		end   = GDL_MIN (gene->cdsStart, gene->exonEnds[0]);
//	}
//	else
//	{
//		start = GDL_MAX (gene->exonStarts[nexon-1], gene->cdsEnd);
//		end   = gene->exonEnds[nexon-1];
//	}
//
//	if (end > start)
//	{
//		return end-start;
//	}
//	else
//		return 0;
//}

//static long
//gdl_igdb_cis_annot_notcds_exon_psize (const void * vtype, const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t gene_idx)
//{
//	size_t i;
//	const gdl_eqtl_gene * gene = chrom->genes[gene_idx];
//	long exon_size = 0;
//	long other_exon = gdl_igdb_cis_annot_cds_exon_psize (vtype, annot, chrom, gene_idx);
//   other_exon += gdl_igdb_cis_annot_first_exon_psize (vtype, annot, chrom, gene_idx);
//   other_exon += gdl_igdb_cis_annot_last_exon_psize (vtype, annot, chrom, gene_idx);
//
//   for(i = 0; i < gene->exonCount; i++)
//   {
//   	exon_size += gene->exonEnds[i]-gene->exonStarts[i];
//   }
//
//   return exon_size-other_exon;
//}

/**
 * bin:10-12,14,30
 */
static size_t *
_parse_bin_range (const gdl_string * token, size_t * nbin)
{
	const size_t len = strlen(token);
	int b1,b2;
	size_t i,j,k,n,*bidx;
	gdl_string * tmp;

	for(n=i=j=0;i<len;)
	{
		//printf (">%s\n",&token[i]);
		for(b1=-1,k=i;j<len && token[j] != ',';j++)
		{
			if (token[j]=='-')
			{
				tmp = gdl_string_alloc (j-k);
				strncpy(tmp,&token[k],j-k);
				b1 = atoi(tmp);
				gdl_string_free (tmp);
				k=j+1;
			}
		}
		tmp = gdl_string_alloc (j-k);
		strncpy(tmp,&token[k],j-k);
		b2 = atoi(tmp);
		gdl_string_free (tmp);
		//printf ("i=%d j=%d b1=%d b2=%d\n", i, j, b1, b2);
		if (b1 != -1)
		{
			n += b2-b1+1;
		}
		else n++;
		i=++j;
	}

	*nbin = n;
	bidx  = GDL_CALLOC (size_t, n);

	for(n=i=j=0;i<len;)
	{
		for(b1=-1,k=i;j<len && token[j] != ',';j++)
		{
			if (token[j]=='-')
			{
				tmp = gdl_string_alloc (j-k);
				strncpy(tmp,&token[k],j-k);
				b1 = atoi(tmp);
				gdl_string_free (tmp);
				k=j+1;
			}
		}
		tmp = gdl_string_alloc (j-k);
		strncpy(tmp,&token[k],j-k);
		b2 = atoi(tmp);
		gdl_string_free (tmp);
		if (b1 != -1)
		{
			for(k=b1;k<=b2;k++,n++)
				bidx[n] = k;
		}
		else
		{
			bidx[n++]=b2;
		}
		i=++j;
	}

	//for(i=0;i<*nbin;i++) printf ("%d %d\n", i, bidx[i]);

	return bidx;
}

/**
 * TSS:-1000,1000;TES:-1000,0
 */
static gdl_boolean
_parse_anchor_ranges (const gdl_string * token, gdl_igdb_cis_annot_track_type * T)
{
	const size_t len = strlen(token);
	size_t i,j,k,n;
	gdl_string * tmp, * buf;
	const gdl_expr_feature_anchor * A;

	for(n=i=0;i<len;i++)
	{
		n+=(token[i]==':');
	}
	if (!n)
	{
		return gdl_false;
	}

	T->nanchor = n;
	T->anchors = GDL_MALLOC (gdl_expr_feature_anchor *, n);
	T->anchor_ranges = GDL_MATRIX_ALLOC (long, n, 2);

	tmp = gdl_string_alloc (len);

	for(n=i=j=0;i<len;i++)
	{
		if (token[i]==':')
		{
			A = gdl_expr_feature_anchor_lookup_acronym (tmp);
			if (!A)
			{
				GDL_ERROR_VAL (gdl_string_sprintf ("Invalid anchor name: %s\n", tmp), 0, GDL_FAILURE);
			}
			T->anchors[n] = gdl_expr_feature_anchor_alloc (A);
			// clean buffer
			for(k = 0; k < j; k++) tmp[k]='\0';
			j=0;
		}
		else if (token[i]==';')
		{
			for(k = 0; k < j && tmp[k]!=','; k++);
			if (k == j)
			{
				GDL_ERROR_VAL (gdl_string_sprintf ("Invalid anchor range [%s] for anchor %s\n", tmp, A->acronym), 0, GDL_FAILURE);
			}
			buf = gdl_string_alloc (k);
			strncpy (buf, tmp, k);
			T->anchor_ranges[n][0] = atol(buf);
			gdl_string_free (buf);
			buf = gdl_string_alloc (j-k-1);
			strncpy (buf, &tmp[k+1], j-k-1);
			T->anchor_ranges[n][1] = atol(buf);
			gdl_string_free (buf);
			// increment anchor ix
			n++;
			// clean buffer
			for(k = 0; k < j; k++) tmp[k]='\0';
			j=0;
		}
		else
		{
			tmp[j++]=token[i];
		}
	}
	// split tmp (,)
	for(k = 0; k < j && tmp[k]!=','; k++);
	if (k == j)
	{
		GDL_ERROR_VAL (gdl_string_sprintf ("Invalid anchor range [%s] for anchor %s\n", tmp, A->acronym), 0, GDL_FAILURE);
	}
	buf = gdl_string_alloc (k);
	strncpy (buf, tmp, k);
	T->anchor_ranges[n][0] = atol(buf);
	gdl_string_free (buf);
	buf = gdl_string_alloc (j-k-1);
	strncpy (buf, &tmp[k+1], j-k-1);
	T->anchor_ranges[n][1] = atol(buf);
	gdl_string_free (buf);
	gdl_string_free (tmp);

	return gdl_true;
}

gdl_igdb_cis_annot_track *
gdl_igdb_cis_annot_track_sscanf (const gdl_string * string)
{
	size_t i,j,n=strlen(string);
	gdl_string * tok;
	const gdl_igdb_cis_annot_track_type * T;
	gdl_igdb_cis_annot_track * t;

	//printf ("TRACK SSCANF [%s]\n", string);

	i=j=0;
	// first, the type of annotation
	tok = gdl_string_next_token (string, n, &i, &j);
	if (!strcmp(gdl_igdb_cis_annot_genome_track->name, tok))
	{
		T=gdl_igdb_cis_annot_genome_track;
	}
	else if (!strcmp(gdl_igdb_cis_annot_genome_score->name, tok))
	{
		T=gdl_igdb_cis_annot_genome_score;
	}
	else if (!strcmp(gdl_igdb_cis_annot_snp_table->name, tok))
	{
		T=gdl_igdb_cis_annot_snp_table;
	}
	else if (!strcmp(gdl_igdb_cis_annot_intron->name, tok))
	{
		T=gdl_igdb_cis_annot_intron;
		t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
		gdl_string_free (tok);
		return t;
	}
	else if (!strcmp(gdl_igdb_cis_annot_cds_intron->name, tok))
	{
		T=gdl_igdb_cis_annot_cds_intron;
		t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
		gdl_string_free (tok);
		return t;
	}
	else if (!strcmp(gdl_igdb_cis_annot_notcds_intron->name, tok))
	{
		T=gdl_igdb_cis_annot_notcds_intron;
		t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
		gdl_string_free (tok);
		return t;
	}
	else if (!strcmp(gdl_igdb_cis_annot_first_intron->name, tok))
	{
		T=gdl_igdb_cis_annot_first_intron;
		t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
		gdl_string_free (tok);
		return t;
	}
	else if (!strcmp(gdl_igdb_cis_annot_last_intron->name, tok))
	{
		T=gdl_igdb_cis_annot_last_intron;
		t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
		gdl_string_free (tok);
		return t;
	}
	else if (!strcmp(gdl_igdb_cis_annot_exon->name, tok))
	{
		T=gdl_igdb_cis_annot_exon;
		t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
		gdl_string_free (tok);
		return t;
	}
	else if (!strcmp(gdl_igdb_cis_annot_cds_exon->name, tok))
	{
		T=gdl_igdb_cis_annot_cds_exon;
		t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
		gdl_string_free (tok);
		return t;
	}
	else if (!strcmp(gdl_igdb_cis_annot_notcds_exon->name, tok))
	{
		T=gdl_igdb_cis_annot_notcds_exon;
		t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
		gdl_string_free (tok);
		return t;
	}
	else if (!strcmp(gdl_igdb_cis_annot_first_exon->name, tok))
	{
		T=gdl_igdb_cis_annot_first_exon;
		t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
		gdl_string_free (tok);
		return t;
	}
	else if (!strcmp(gdl_igdb_cis_annot_last_exon->name, tok))
	{
		T=gdl_igdb_cis_annot_last_exon;
		t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
		gdl_string_free (tok);
		return t;
	}
	else if (!strcmp(gdl_igdb_cis_annot_5_utr->name, tok))
	{
		T=gdl_igdb_cis_annot_5_utr;
		t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
		gdl_string_free (tok);
		return t;
	}
	else if (!strcmp(gdl_igdb_cis_annot_3_utr->name, tok))
	{
		T=gdl_igdb_cis_annot_3_utr;
		t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
		gdl_string_free (tok);
		return t;
	}
	else if (!strcmp(gdl_igdb_cis_annot_inside_gene->name, tok))
	{
		T=gdl_igdb_cis_annot_inside_gene;
		t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
		gdl_string_free (tok);
		return t;
	}
	else if (!strcmp(gdl_igdb_cis_annot_inside_feature->name, tok))
	{
		T=gdl_igdb_cis_annot_inside_feature;
		t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
		gdl_string_free (tok);
		return t;
	}
	else if (!strcmp(gdl_igdb_cis_annot_splice_site->name, tok))
	{
		T=gdl_igdb_cis_annot_splice_site;
		t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
		gdl_string_free (tok);
		return t;
	}
	else
	{
		//gdl_string_free (tok);
		GDL_ERROR_VAL (gdl_string_sprintf ("Invalid annotation type %s", tok), GDL_EINVAL, 0);
	}
	gdl_string_free (tok);
	// then, the name of the annotation
	tok = gdl_string_next_token (string, n, &i, &j);
	t = gdl_igdb_cis_annot_track_alloc (T, tok, 1);
	gdl_string_free (tok);
	// follows the directory
	t->dir = gdl_string_next_token (string, n, &i, &j);
	if (T == gdl_igdb_cis_annot_snp_table)
	{
		FILE * stream;
		stream = gdl_fileopen (t->dir, "r");
		t->T->extra = gdl_snp_table_fscanf (stream);
		gdl_fileclose (t->dir, stream);
		return t;
	}
//	else
//	{
//		tok = gdl_string_next_token (string, n, &i, &j);
//		if (!strcmp(tok, "inside"))
//		{
//			if (!strcmp(t->T->name == gdl_igdb_cis_annot_genome_track)
//			{
//				t->T->snp_value=&gdl_igdb_cis_annot_genome_track_inside;
//			}
//			else
//			{
//				t->T->snp_value=&gdl_igdb_cis_annot_genome_score_inside;
//			}
//		}
//		else
//		{
//			//gdl_string_free (tok);
//			//gdl_igdb_cis_annot_track_free (t);
//			GDL_ERROR_VAL (gdl_string_sprintf ("Invalid annotation type %s", tok), GDL_EINVAL, 0);
//		}
//		gdl_string_free (tok);
//	}
	if (j < n)
	{
		// last token: the gene only flag or the anchor locations
		tok = gdl_string_next_token (string, n, &i, &j);
		if (!_parse_anchor_ranges (tok, t->T))
		{
			t->T->gene_only = (size_t)atoi(tok);
			gdl_string_free (tok);
		}
	}
	return t;
}

gdl_igdb_cis_annot_track *
gdl_igdb_cis_annot_track_fread (FILE * stream)
{
	if (stream)
	{
		int status;
		gdl_string * tok;
		const gdl_igdb_cis_annot_track_type * T;
		gdl_igdb_cis_annot_track * t;

		tok = gdl_string_fread (stream);
		GDL_FREAD_STATUS (tok!=0,1);
		if (!strcmp(gdl_igdb_cis_annot_genome_track->name, tok))
		{
			T=gdl_igdb_cis_annot_genome_track;
		}
		else if (!strcmp(gdl_igdb_cis_annot_genome_score->name, tok))
		{
			T=gdl_igdb_cis_annot_genome_track;
		}
		else if (!strcmp(gdl_igdb_cis_annot_snp_table->name, tok))
		{
			T=gdl_igdb_cis_annot_snp_table;
		}
		else if (!strcmp(gdl_igdb_cis_annot_intron->name, tok))
		{
			T=gdl_igdb_cis_annot_intron;
			t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
			gdl_string_free (tok);
			return t;
		}
		else if (!strcmp(gdl_igdb_cis_annot_cds_intron->name, tok))
		{
			T=gdl_igdb_cis_annot_cds_intron;
			t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
			gdl_string_free (tok);
			return t;
		}
		else if (!strcmp(gdl_igdb_cis_annot_notcds_intron->name, tok))
		{
			T=gdl_igdb_cis_annot_notcds_intron;
			t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
			gdl_string_free (tok);
			return t;
		}
		else if (!strcmp(gdl_igdb_cis_annot_first_intron->name, tok))
		{
			T=gdl_igdb_cis_annot_first_intron;
			t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
			gdl_string_free (tok);
			return t;
		}
		else if (!strcmp(gdl_igdb_cis_annot_last_intron->name, tok))
		{
			T=gdl_igdb_cis_annot_last_intron;
			t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
			gdl_string_free (tok);
			return t;
		}
		else if (!strcmp(gdl_igdb_cis_annot_exon->name, tok))
		{
			T=gdl_igdb_cis_annot_exon;
			t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
			gdl_string_free (tok);
			return t;
		}
		else if (!strcmp(gdl_igdb_cis_annot_cds_exon->name, tok))
		{
			T=gdl_igdb_cis_annot_cds_exon;
			t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
			gdl_string_free (tok);
			return t;
		}
		else if (!strcmp(gdl_igdb_cis_annot_notcds_exon->name, tok))
		{
			T=gdl_igdb_cis_annot_notcds_exon;
			t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
			gdl_string_free (tok);
			return t;
		}
		else if (!strcmp(gdl_igdb_cis_annot_first_exon->name, tok))
		{
			T=gdl_igdb_cis_annot_first_exon;
			t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
			gdl_string_free (tok);
			return t;
		}
		else if (!strcmp(gdl_igdb_cis_annot_last_exon->name, tok))
		{
			T=gdl_igdb_cis_annot_last_exon;
			t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
			gdl_string_free (tok);
			return t;
		}
		else if (!strcmp(gdl_igdb_cis_annot_5_utr->name, tok))
		{
			T=gdl_igdb_cis_annot_5_utr;
			t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
			gdl_string_free (tok);
			return t;
		}
		else if (!strcmp(gdl_igdb_cis_annot_3_utr->name, tok))
		{
			T=gdl_igdb_cis_annot_3_utr;
			t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
			gdl_string_free (tok);
			return t;
		}
		else if (!strcmp(gdl_igdb_cis_annot_inside_gene->name, tok))
		{
			T=gdl_igdb_cis_annot_inside_gene;
			t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
			gdl_string_free (tok);
			return t;
		}
		else if (!strcmp(gdl_igdb_cis_annot_inside_feature->name, tok))
		{
			T=gdl_igdb_cis_annot_inside_feature;
			t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
			gdl_string_free (tok);
			return t;
		}
		else if (!strcmp(gdl_igdb_cis_annot_splice_site->name, tok))
		{
			T=gdl_igdb_cis_annot_splice_site;
			t = gdl_igdb_cis_annot_track_alloc (T, T->name, 1);
			gdl_string_free (tok);
			return t;
		}
		gdl_string_free (tok);

		tok = gdl_string_fread (stream);
		GDL_FREAD_STATUS (tok!=0,1);

		t = gdl_igdb_cis_annot_track_alloc (T, tok, 1);

		gdl_string_free (tok);

		t->dir = gdl_string_fread (stream);
		GDL_FREAD_STATUS (t->dir!=0,1);

		if (!strcmp (t->T->name, gdl_igdb_cis_annot_snp_table->name))
		{
			FILE * gstream;
			gstream = gdl_fileopen (t->dir, "r");
			t->T->extra = gdl_snp_table_fscanf (gstream);
			gdl_fileclose (t->dir, gstream);
			return t;
		}
		else if (!strcmp (t->T->name, gdl_igdb_cis_annot_genome_track->name))
		{
			tok = gdl_string_fread (stream);
			GDL_FREAD_STATUS (tok!=0,1);
			if (!strcmp(tok, "inside"))
			{
				t->T->snp_value=&gdl_igdb_cis_annot_genome_track_inside;
			}
//			else if (!strcmp(tok, "between"))
//			{
//				t->T->snp_value=&gdl_igdb_cis_annot_genome_track_between;
//			}
			gdl_string_free (tok);
		}
		else if (!strcmp (t->T->name, gdl_igdb_cis_annot_genome_score->name))
		{
			tok = gdl_string_fread (stream);
			GDL_FREAD_STATUS (tok!=0,1);
			if (!strcmp(tok, "inside"))
			{
				t->T->snp_value=&gdl_igdb_cis_annot_genome_score_inside;
			}
//			else if (!strcmp(tok, "between"))
//			{
//				t->T->snp_value=&gdl_igdb_cis_annot_genome_track_between;
//			}
			gdl_string_free (tok);
		}
//		else if (!strcmp (t->T->name, gdl_igdb_cis_annot_snp_track->name))
//		{
//			t->T->snp_value=&gdl_igdb_cis_annot_snp_track_value;
//		}
		status = fread (&(t->T->gene_only), sizeof(size_t), 1, stream);
		GDL_FREAD_STATUS (status, 1);
		status = fread (&(t->T->bin_only), sizeof(size_t), 1, stream);
		GDL_FREAD_STATUS (status, 1);
		if (t->T->bin_only)
		{
			t->T->bin_idx = GDL_MALLOC (size_t, t->T->bin_only);
			status = fread (t->T->bin_idx, sizeof(size_t), t->T->bin_only, stream);
			GDL_FREAD_STATUS (status, t->T->bin_only);
		}

		return t;
	}
	return 0;
}

int
gdl_igdb_cis_annot_track_fwrite (FILE * stream, const gdl_igdb_cis_annot_track * t)
{
	if (stream && t)
	{
		int status;

		status = gdl_string_fwrite (stream, t->T->name);
		GDL_FWRITE_STATUS (status, GDL_SUCCESS);

		if (!strcmp(gdl_igdb_cis_annot_intron->name, t->T->name))
			return GDL_SUCCESS;
		if (!strcmp(gdl_igdb_cis_annot_cds_intron->name, t->T->name))
			return GDL_SUCCESS;
		if (!strcmp(gdl_igdb_cis_annot_notcds_intron->name, t->T->name))
			return GDL_SUCCESS;
		if (!strcmp(gdl_igdb_cis_annot_first_intron->name, t->T->name))
			return GDL_SUCCESS;
		if (!strcmp(gdl_igdb_cis_annot_last_intron->name, t->T->name))
			return GDL_SUCCESS;
		if (!strcmp(gdl_igdb_cis_annot_exon->name, t->T->name))
			return GDL_SUCCESS;
		if (!strcmp(gdl_igdb_cis_annot_cds_exon->name, t->T->name))
			return GDL_SUCCESS;
		if (!strcmp(gdl_igdb_cis_annot_notcds_exon->name, t->T->name))
			return GDL_SUCCESS;
		if (!strcmp(gdl_igdb_cis_annot_first_exon->name, t->T->name))
			return GDL_SUCCESS;
		if (!strcmp(gdl_igdb_cis_annot_last_exon->name, t->T->name))
			return GDL_SUCCESS;
		if (!strcmp(gdl_igdb_cis_annot_5_utr->name, t->T->name))
			return GDL_SUCCESS;
		if (!strcmp(gdl_igdb_cis_annot_3_utr->name, t->T->name))
			return GDL_SUCCESS;
		if (!strcmp(gdl_igdb_cis_annot_inside_gene->name, t->T->name))
			return GDL_SUCCESS;
		if (!strcmp(gdl_igdb_cis_annot_inside_feature->name, t->T->name))
			return GDL_SUCCESS;
		if (!strcmp(gdl_igdb_cis_annot_splice_site->name, t->T->name))
			return GDL_SUCCESS;

		status = gdl_string_fwrite (stream, t->name);
		GDL_FWRITE_STATUS (status, GDL_SUCCESS);
		status = gdl_string_fwrite (stream, t->dir);
		GDL_FWRITE_STATUS (status, GDL_SUCCESS);
		if (!strcmp(gdl_igdb_cis_annot_snp_table->name, t->T->name))
		{
			return GDL_SUCCESS;
		}
		if (t->T->snp_value==&gdl_igdb_cis_annot_genome_track_inside)
		{
			status = gdl_string_fwrite (stream, "inside");
			GDL_FWRITE_STATUS (status, GDL_SUCCESS);
		}
		if (t->T->snp_value==&gdl_igdb_cis_annot_genome_score_inside)
		{
			status = gdl_string_fwrite (stream, "inside");
			GDL_FWRITE_STATUS (status, GDL_SUCCESS);
		}
//		else if (t->T->snp_value==&gdl_igdb_cis_annot_genome_track_between)
//		{
//			status = gdl_string_fwrite (stream, "between");
//			GDL_FWRITE_STATUS (status, GDL_SUCCESS);
//		}
		status = fwrite (&(t->T->gene_only), sizeof(size_t), 1, stream);
		GDL_FWRITE_STATUS (status, 1);
		status = fwrite (&(t->T->bin_only), sizeof(size_t), 1, stream);
		GDL_FWRITE_STATUS (status, 1);
		if (t->T->bin_only)
		{
			status = fwrite (t->T->bin_idx, sizeof(size_t), t->T->bin_only, stream);
			GDL_FWRITE_STATUS (status, t->T->bin_only);
		}

		return GDL_SUCCESS;
	}
	return GDL_EINVAL;
}

const static gdl_igdb_cis_annot_track_type GENOME_TRACK =
{
	"genome_track",
	0,
	0,
	&gdl_igdb_cis_annot_chromosome_fscanf3,
	&gdl_igdb_cis_annot_genome_track_inside,
	&gdl_igdb_cis_annot_genome_track_psize,
	0,
	0,
	0,
	0
};

const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_genome_track = &GENOME_TRACK;

const static gdl_igdb_cis_annot_track_type GENOME_SCORE =
{
	"genome_score",
	0,
	0,
	&gdl_igdb_cis_annot_chromosome_fscanf4,
	&gdl_igdb_cis_annot_genome_score_inside,
	&gdl_igdb_cis_annot_genome_track_psize,
	0,
	0,
	0,
	0
};

const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_genome_score = &GENOME_SCORE;

const static gdl_igdb_cis_annot_track_type SNP_TABLE =
{
	"snp_table",
	0,
	0,
	0,
	&gdl_igdb_cis_annot_snp_table_value,
	0,
	0,
	0,
	0,
	0
};

const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_snp_table = &SNP_TABLE;

//const static gdl_igdb_cis_annot_track_type GENE_BETWEEN =
//{
//	"gene_between",
//	0,
//	0,
//	0,
//	&gdl_igdb_cis_annot_gene_between_value,
//	0,
//	0,
//	0,
//	0
//};
//
//const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_gene_between = &GENE_BETWEEN;

const static gdl_igdb_cis_annot_track_type INTRON =
{
	"intron",
	0,
	0,
	0,
	&gdl_igdb_cis_annot_snp_intron,
	0,
	0,
	0,
	0,
	0
};

const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_intron = &INTRON;

const static gdl_igdb_cis_annot_track_type CDS_INTRON =
{
	"cds_intron",
	0,
	0,
	0,
	&gdl_igdb_cis_annot_snp_cds_intron,
	&gdl_igdb_cis_annot_cds_intron_psize,
	0,
	0,
	0,
	0
};

const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_cds_intron = &CDS_INTRON;

const static gdl_igdb_cis_annot_track_type NOTCDS_INTRON =
{
	"non_cds_intron",
	0,
	0,
	0,
	&gdl_igdb_cis_annot_snp_notcds_intron,
	0,
	//&gdl_igdb_cis_annot_notcds_intron_psize,
	0,
	0,
	0
};

const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_notcds_intron = &NOTCDS_INTRON;

const static gdl_igdb_cis_annot_track_type FIRST_INTRON =
{
	"first_intron",
	0,
	0,
	0,
	&gdl_igdb_cis_annot_snp_first_intron,
	0,
	//&gdl_igdb_cis_annot_first_intron_psize,
	0,
	0,
	0
};

const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_first_intron = &FIRST_INTRON;

const static gdl_igdb_cis_annot_track_type LAST_INTRON =
{
	"last_intron",
	0,
	0,
	0,
	&gdl_igdb_cis_annot_snp_last_intron,
	0,
	//&gdl_igdb_cis_annot_last_intron_psize,
	0,
	0,
	0
};

const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_last_intron = &LAST_INTRON;

const static gdl_igdb_cis_annot_track_type EXON =
{
	"exon",
	0,
	0,
	0,
	&gdl_igdb_cis_annot_snp_exon,
	0,
	0,
	0,
	0,
};

const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_exon = &EXON;

const static gdl_igdb_cis_annot_track_type CDS_EXON =
{
	"cds_exon",
	0,
	0,
	0,
	&gdl_igdb_cis_annot_snp_cds_exon,
	0,
	//&gdl_igdb_cis_annot_cds_exon_psize,
	0,
	0,
	0
};

const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_cds_exon = &CDS_EXON;

const static gdl_igdb_cis_annot_track_type NOTCDS_EXON =
{
	"non_cds_exon",
	0,
	0,
	0,
	&gdl_igdb_cis_annot_snp_notcds_exon,
	0,
	//&gdl_igdb_cis_annot_notcds_exon_psize,
	0,
	0,
	0
};

const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_notcds_exon = &NOTCDS_EXON;

//const static gdl_igdb_cis_annot_track_type CODING_SYN_EXON =
//{
//	"coding_syn_exon",
//	0,
//	0,
//	0,
//	&gdl_igdb_cis_annot_snp_coding_syn_exon,
//	&gdl_igdb_cis_annot_cds_exon_psize,
//	0,
//	0,
//	0
//};
//
//const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_coding_syn_exon = &CODING_SYN_EXON;
//
//const static gdl_igdb_cis_annot_track_type CODING_NON_EXON =
//{
//	"coding_non_exon",
//	0,
//	0,
//	0,
//	&gdl_igdb_cis_annot_snp_coding_non_exon,
//	&gdl_igdb_cis_annot_cds_exon_psize,
//	0,
//	0,
//	0
//};
//
//const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_coding_non_exon = &CODING_NON_EXON;

const static gdl_igdb_cis_annot_track_type FIRST_EXON =
{
	"first_exon",
	0,
	0,
	0,
	&gdl_igdb_cis_annot_snp_first_exon,
	0,
	//&gdl_igdb_cis_annot_first_exon_psize,
	0,
	0,
	0
};

const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_first_exon = &FIRST_EXON;

const static gdl_igdb_cis_annot_track_type LAST_EXON =
{
	"last_exon",
	0,
	0,
	0,
	&gdl_igdb_cis_annot_snp_last_exon,
	0,
	//&gdl_igdb_cis_annot_last_exon_psize,
	0,
	0,
	0
};

const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_last_exon = &LAST_EXON;

const static gdl_igdb_cis_annot_track_type UTR5 =
{
	"5_utr",
	0,
	0,
	0,
	&gdl_igdb_cis_annot_snp_5_utr,
	0,
	0,
	0,
	0
};

const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_5_utr = &UTR5;

const static gdl_igdb_cis_annot_track_type UTR3 =
{
	"3_utr",
	0,
	0,
	0,
	&gdl_igdb_cis_annot_snp_3_utr,
	0,
	0,
	0,
	0
};

const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_3_utr = &UTR3;

const static gdl_igdb_cis_annot_track_type INSIDE_GENE =
{
	"inside_gene",
	0,
	0,
	0,
	&gdl_igdb_cis_annot_snp_inside_gene,
	0,
	0,
	0,
	0
};

const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_inside_gene = &INSIDE_GENE;

const static gdl_igdb_cis_annot_track_type INSIDE_FEATURE =
{
	"inside_feature",
	0,
	0,
	0,
	&gdl_igdb_cis_annot_snp_inside_feature,
	0,
	0,
	0,
	0
};

const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_inside_feature = &INSIDE_FEATURE;

const static gdl_igdb_cis_annot_track_type SPLICE_SITE =
{
	"splice_site",
	0,
	0,
	0,
	&gdl_igdb_cis_annot_snp_splice_site,
	0,
	0,
	0,
	0
};

const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_splice_site = &SPLICE_SITE;

static void
cluster_track_by_repository (gdl_igdb_cis_annot * a)
{
	size_t i;
	size_t * rep_idx;
	gdl_hashtable * rep_hash;

	rep_hash = gdl_hashtable_alloc (gdl_interface_uint, 0);

	for(i = 0; i < a->size; i++)
	{
		if (a->tracks[i]->dir)
		{
			rep_idx = gdl_hashtable_lookup (rep_hash, a->tracks[i]->dir);
			if (!rep_idx)
			{
				rep_idx = GDL_CALLOC (size_t, a->size);
				gdl_hashtable_add (rep_hash, a->tracks[i]->dir, rep_idx, 0);
			}
			rep_idx[rep_idx[0]+1]=i;
			rep_idx[0]++;
		}
	}

	a->nrep = gdl_hashtable_size (rep_hash);
	GDL_FREE (a->rep_idx);

	if (a->nrep)
	{
		gdl_hashtable_itr * rep_itr = gdl_hashtable_iterator (rep_hash);
		a->rep_idx = GDL_MALLOC (size_t *, a->nrep);
		i=0;
		do
		{
			rep_idx       = gdl_hashtable_iterator_value (rep_itr);
			a->rep_idx[i] = rep_idx;
			i++;
		}
		while(gdl_hashtable_iterator_next (rep_itr));
		gdl_hashtable_iterator_free (rep_itr);
	}
	gdl_hashtable_free (rep_hash);
}

gdl_igdb_cis_annot *
gdl_igdb_cis_annot_fscanf (FILE * stream)
{
	size_t i,n;
	gdl_string * line=0;
	gdl_igdb_cis_annot * a;
	gdl_list * buff = gdl_list_alloc (gdl_list_default);

	while(gdl_getline (&line, &n, stream) != -1)
	{
		gdl_igdb_cis_annot_track * t = gdl_igdb_cis_annot_track_sscanf (line);
		gdl_list_push_back (buff, t, 0);
		gdl_string_free (line); line=0;
	}

	a = GDL_CALLOC (gdl_igdb_cis_annot, 1);

	a->size = gdl_list_size (buff);

	if (a->size)
	{
		gdl_list_itr * itr = gdl_list_iterator_front (buff);

		a->tracks = GDL_MALLOC (gdl_igdb_cis_annot_track *, a->size);
		i=0;
		do
		{
			a->tracks[i]=gdl_list_iterator_value(itr);
			i++;
		}
		while(gdl_list_iterator_next (itr));
		gdl_list_iterator_free (itr);

		cluster_track_by_repository (a);
	}
	else
	{
		GDL_FREE (a);
		a=0;
	}

	gdl_list_free (buff);

	return a;
}

gdl_igdb_cis_annot *
gdl_igdb_cis_annot_sscanf (const gdl_string * single)
{
	size_t i,n;
	gdl_string * line=0;
	gdl_igdb_cis_annot * a;

	a            = GDL_CALLOC (gdl_igdb_cis_annot, 1);
	a->size      = 1;
	a->tracks    = GDL_MALLOC (gdl_igdb_cis_annot_track *, a->size);
	a->tracks[0] = gdl_igdb_cis_annot_track_sscanf (single);

	cluster_track_by_repository (a);

	return a;
}

gdl_igdb_cis_annot *
gdl_igdb_cis_annot_alloc (void)
{
	gdl_igdb_cis_annot * a;

	a = GDL_CALLOC (gdl_igdb_cis_annot, 1);

	return a;
}

int
gdl_igdb_cis_annot_nested (gdl_igdb_cis_annot * main, gdl_igdb_cis_annot * sub)
{
	if (main->nnested)
	{
		gdl_igdb_cis_annot ** tmp = GDL_MALLOC (gdl_igdb_cis_annot *, main->nnested+1);
		memcpy(tmp,  main->nested, sizeof(gdl_igdb_cis_annot *)*main->nnested);
		GDL_FREE (main->nested);
		main->nested = tmp;
	}
	else
	{
		main->nested = GDL_MALLOC (gdl_igdb_cis_annot *, 1);
	}
	main->nested[main->nnested] = sub;
	return ++(main->nnested);
}

static int
_gdl_igdb_cis_annot_unnested (gdl_igdb_cis_annot_track ** t, size_t * i, gdl_igdb_cis_annot * nested)
{
	size_t j;

	if (nested->nnested)
	{
		for(j = 0; j < nested->nnested; j++)
		{
			_gdl_igdb_cis_annot_unnested (t, i, nested->nested[j]);
		}
	}
	else
	{
		for(j = 0; j < nested->size; j++)
		{
			t[(*i)++] = nested->tracks[j];
		}
	}
}

void
gdl_igdb_cis_annot_unnested (gdl_igdb_cis_annot * main)
{
	if (main->nnested)
	{
		size_t i, j;

		main->size   = gdl_igdb_cis_annot_size (main);
		main->tracks = GDL_MALLOC (gdl_igdb_cis_annot_track *, main->size);
		for(i = j = 0; i < main->nnested; i++)
		{
			_gdl_igdb_cis_annot_unnested (main->tracks, &j, main->nested[i]);
		}
		main->nested = 0;
		GDL_FREE (main->nested);
		main->nnested = 0;
	}
}

size_t
gdl_igdb_cis_annot_size (const gdl_igdb_cis_annot * a)
{
	size_t n = 0;

	if (a->nnested)
	{
		size_t i;
		for(i = 0; i < a->nnested; i++)
		{
			n += gdl_igdb_cis_annot_size (a->nested[i]);
		}
	}

	n += a->size;

	return n;
}

void
gdl_igdb_cis_annot_free (gdl_igdb_cis_annot * a)
{
	if (a)
	{
		size_t i;
		for (i = 0; i < a->size; i++)
		{
			gdl_igdb_cis_annot_track_free (a->tracks[i]);
		}
		GDL_FREE (a->tracks);
//		for(i = 0; i < a->nnested; i++)
//		{
//			gdl_igdb_cis_annot_free (a->nested[i]);
//		}
		GDL_FREE (a->nested);
		GDL_FREE (a);
	}
}

gdl_igdb_cis_annot *
gdl_igdb_cis_annot_fread (FILE * stream)
{
	if (stream)
	{
		int status;
		size_t i;
		gdl_igdb_cis_annot * a;

		a = GDL_CALLOC (gdl_igdb_cis_annot, 1);
		status = fread (&a->size, sizeof(size_t), 1, stream);
		GDL_FREAD_STATUS (status, 1);
		a->tracks = GDL_MALLOC (gdl_igdb_cis_annot_track *, a->size);
		for(i = 0; i < a->size; i++)
		{
			a->tracks[i]=gdl_igdb_cis_annot_track_fread (stream);
			GDL_FREAD_STATUS (a->tracks[i]!=0,1);
		}

		cluster_track_by_repository (a);

		return a;
	}
	return 0;
}

int
gdl_igdb_cis_annot_fwrite (FILE * stream, const gdl_igdb_cis_annot * a)
{
	if (stream && a)
	{
		int status;
		size_t i;

		status = fwrite (&a->size, sizeof(size_t), 1, stream);
		GDL_FWRITE_STATUS (status, 1);
		for(i = 0; i < a->size; i++)
		{
			status=gdl_igdb_cis_annot_track_fwrite (stream, a->tracks[i]);
			GDL_FWRITE_STATUS (status, GDL_SUCCESS);
		}
		return GDL_SUCCESS;
	}
	return GDL_EINVAL;
}

gdl_igdb_cis_annot *
gdl_igdb_cis_annot_nested_fread (FILE * stream)
{
	if (stream)
	{
		int status;
		size_t i;
		gdl_igdb_cis_annot * a;

		a = GDL_CALLOC (gdl_igdb_cis_annot, 1);
		status = fread (&a->nnested, sizeof(size_t), 1, stream);
		GDL_FREAD_STATUS (status, 1);
		a->nested = GDL_MALLOC (gdl_igdb_cis_annot *, a->nnested);
		for(i = 0; i < a->nnested; i++)
		{
			a->nested[i]=gdl_igdb_cis_annot_fread (stream);
			GDL_FREAD_STATUS (a->nested[i]!=0,1);
		}
		return a;
	}
	return 0;
}

int
gdl_igdb_cis_annot_nested_fwrite (FILE * stream, const gdl_igdb_cis_annot * a)
{
	if (stream && a)
	{
		int status;
		size_t i;

		status = fwrite (&a->nnested, sizeof(size_t), 1, stream);
		GDL_FWRITE_STATUS (status, 1);
		for(i = 0; i < a->nnested; i++)
		{
			status = gdl_igdb_cis_annot_fwrite (stream, a->nested[i]);
			GDL_FWRITE_STATUS (status, GDL_SUCCESS);
		}
		return GDL_SUCCESS;
	}
	return GDL_EINVAL;
}

int
gdl_igdb_cis_annot_load (const gdl_igdb_cis_annot * a, const gdl_igdb_chromosome * chrom)
{
	size_t r;

	if (a->nnested)
	{
		size_t i;
		for(i = 0; i < a->nnested; i++)
		{
			gdl_igdb_cis_annot_load (a->nested[i], chrom);
		}
	}
	else
	{
		if (a->nrep) // efficient way... in term of mem. usage
		{
			for(r = 0;r < a->nrep; r++)
			{
				size_t j;
				size_t nr = a->rep_idx[r][0];
				size_t r0 = a->rep_idx[r][1];
				// load r0;
				if (gdl_igdb_cis_annot_track_load (a->tracks[r0], chrom)!=GDL_SUCCESS)
				{
					gdl_string * warn_str = gdl_string_sprintf ("Unable to load chromosome track %s for annotation %s (%s)", chrom->name, a->tracks[r0]->name, a->tracks[r0]->T->name);
					GDL_WARNING (warn_str, GDL_CONTINUE);
					gdl_string_free (warn_str);
				}
				for(j = 2; j <= nr; j++)
				{
					//printf (" %d,", a->rep_idx[r][j]);
					a->tracks[a->rep_idx[r][j]]->chrom = a->tracks[r0]->chrom;
				}
				//printf ("\n");
			}
		}
		else // greedy way (old version)
		{
			for(r = 0;r < a->size; r++)
			{
				gdl_igdb_cis_annot_track_load (a->tracks[r], chrom);
			}
		}
	}
}

int
gdl_igdb_cis_annot_unload (const gdl_igdb_cis_annot * a)
{
	size_t r;

	if (a->nnested)
	{
		size_t i;
		for(i = 0; i < a->nnested; i++)
		{
			gdl_igdb_cis_annot_unload (a->nested[i]);
		}
	}
	else
	{
		if (a->nrep) // efficient way... in term of mem. usage
		{
			for(r = 0;r < a->nrep; r++)
			{
				size_t j;
				size_t nr = a->rep_idx[r][0];
				size_t r0 = a->rep_idx[r][1];
				// load r0;
				gdl_igdb_cis_annot_track_unload (a->tracks[r0]);
				for(j = 2; j <= nr; j++)
					a->tracks[a->rep_idx[r][j]]->chrom = 0;
			}
		}
		else // greedy way (old version)
			for(r = 0;r < a->size; r++)
				gdl_igdb_cis_annot_track_unload (a->tracks[r]);
	}
}

size_t
gdl_igdb_cis_annot_focus (const gdl_igdb_cis_annot * a, const long from, const long to, const size_t thread_idx)
{
	size_t r;

	if (a->nnested)
	{
		size_t i;
		for(i = 0; i < a->nnested; i++)
		{
			gdl_igdb_cis_annot_focus (a->nested[i], from, to, thread_idx);
		}
	}
	else
	{
		if (a->nrep) // efficient way... in term of mem. usage
		{
			for(r = 0;r < a->nrep; r++)
			{
				size_t j;
				size_t nr = a->rep_idx[r][0];
				size_t r0 = a->rep_idx[r][1];
				// load r0;
				gdl_igdb_cis_annot_track_focus (a->tracks[r0], &(a->tracks[r0]->focusth[thread_idx]), from, to);
				for(j = 2; j <= nr; j++)
				{
					a->tracks[a->rep_idx[r][j]]->focusth[thread_idx] = a->tracks[r0]->focusth[thread_idx];
				}
			}
		}
		else // greedy way (old version)
		{
			for(r = 0;r < a->size; r++)
			{
				gdl_igdb_cis_annot_track_focus (a->tracks[r], &(a->tracks[r]->focusth[thread_idx]), from, to);
			}
		}
	}
}

void
gdl_igdb_cis_annot_unfocus (const gdl_igdb_cis_annot * a, const size_t thread_idx)
{
	size_t r;

	if (a->nnested)
	{
		size_t i;
		for(i = 0; i < a->nnested; i++)
		{
			gdl_igdb_cis_annot_unfocus (a->nested[i], thread_idx);
		}
	}
	else
	{
		if (a->nrep) // efficient way... in term of mem. usage
		{
			for(r = 0;r < a->nrep; r++)
			{
				size_t j;
				size_t nr = a->rep_idx[r][0];
				size_t r0 = a->rep_idx[r][1];
				// load r0;
				gdl_igdb_cis_annot_track_unfocus (a->tracks[r0], &(a->tracks[r0]->focusth[thread_idx]));
				for(j = 2; j <= nr; j++)
				{
					a->tracks[a->rep_idx[r][j]]->focusth[thread_idx] = 0;
				}
			}
		}
		else // greedy way (old version)
		{
			for(r = 0;r < a->size; r++)
			{
				gdl_igdb_cis_annot_track_unfocus (a->tracks[r], &(a->tracks[r]->focusth[thread_idx]));
			}
		}
	}
}

void
gdl_igdb_cis_annot_mth (gdl_igdb_cis_annot * a, const size_t nthread)
{
	if (a->nnested)
	{
		size_t i;
		for(i = 0; i < a->nnested; i++)
		{
			gdl_igdb_cis_annot_mth (a->nested[i], nthread);
		}
	}
	else
	{
		size_t r;
		for(r = 0;r < a->size; r++)
		{
			gdl_igdb_cis_annot_track_mth (a->tracks[r], nthread);
		}
	}
}
