/*
 *  expr/fmat.c
 *
 *  $Author: baptiste $, $Date: 2008-05-13 15:33:43 $, $Version$
 *
 *  eQTNMiner : a C package for high-resolution mapping of eQTNs
 *
 *  Copyright (C) 2008  Jean-Baptiste Veyrieras, University of Chicago, USA.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 */
#include <math.h>

#include <getopt.h>
#include <stddef.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>

#include <gdl/gdl_common.h>
#include <gdl/gdl_version.h>
#include <gdl/gdl_errno.h>
#include <gdl/gdl_io.h>
#include <gdl/gdl_math.h>
#include <gdl/gdl_labeled_matrix.h>
#include <gdl/gdl_statistics_double.h>
#include <gdl/gdl_expr_feature.h>
#include <gdl/gdl_igdb_genome.h>
#include <gdl/gdl_igdb_looper.h>

gdl_labeled_matrix *
gdl_igdb_get_fexp_matrix (const gdl_igdb_genome * g,
                          const gdl_boolean qqnorm,
                          const gdl_boolean center,
                          gdl_feature_table * feature_table)
{
	size_t i, j, k, c, f, NN, NF;
	gdl_labeled_matrix * X;
	gdl_igdb_cis_looper * looper;

	looper = gdl_igdb_cis_looper_alloc (g, 0, feature_table, 0, 0, 0, stderr);

	for (NF = NN = c = 0; c < g->nchrom; c++)
	{
		const size_t nf = gdl_igdb_cis_looper_init_chromosome_loop (looper, c);
		for (f = 0; f < nf; f++)
		{
			if (gdl_igdb_cis_looper_init_feature_loop (looper, f))
			{
				(NF)++;
				if (!NN) NN = looper->_chrom->nindiv;
			}
			gdl_igdb_cis_looper_end_feature_loop (looper, f);
		} // loop on feature
		gdl_igdb_cis_looper_end_chromosome_loop (looper, c);
	} // loop on chromosome
	gdl_igdb_cis_looper_free (looper);

	if (NF && NN)
	{
		X = gdl_labeled_matrix_alloc (NF, NN);
		if (g->sample_info)
		{
			for(i = 0; i < NN; i++)
			{
				X->col_names[i] = gdl_string_clone (g->sample_info->row_names[i]);
			}
		}
		else
		{
			for(i = 0; i < NN; i++)
			{
				X->col_names[i] = gdl_string_sprintf ("I_%d", i+1);
			}
		}
	}
	else
	{
		GDL_ERROR_VAL ("ERROR", GDL_EINVAL, 0);
	}

	looper = gdl_igdb_cis_looper_alloc (g, 0, feature_table, 0, 0, 0, stderr);
	for (NF = c = 0; c < g->nchrom; c++)
	{
		const size_t nf = gdl_igdb_cis_looper_init_chromosome_loop (looper, c);
		for (f = 0; f < nf; f++)
		{
			if (gdl_igdb_cis_looper_init_feature_loop (looper, f))
			{
				for(k = i = 0; i < looper->_chrom->npop; i++)
				{
					double * x, mu = 0;
					size_t nn = 0;

					x = looper->_feature->data[i];

					if (qqnorm)
					{
						x = gdl_stats_qqnorm (looper->_feature->data[i], looper->_chrom->pop_sizes[i]);
					}
					if (center)
					{
						mu = gdl_stats_mean (x, 1, looper->_chrom->pop_sizes[i]);
					}
					for(j = 0; j < looper->_chrom->pop_sizes[i]; j++, k++)
					{
						if (gdl_isnan (x[j]))
						{
							gdl_matrix_set (X->X, NF, k, (center) ? 0 : GDL_NAN);
						}
						else
						{
							gdl_matrix_set (X->X, NF, k, x[j] - mu);
						}
					}
					if (qqnorm)
					{
						GDL_FREE (x);
					}
				}
				X->row_names[NF] = gdl_string_clone (looper->_feature->name);
				(NF)++;
			}
			gdl_igdb_cis_looper_end_feature_loop (looper, f);
		} // loop on feature
		gdl_igdb_cis_looper_end_chromosome_loop (looper, c);
	} // loop on chromosome
	gdl_igdb_cis_looper_free (looper);


	return X;
}
