/*
 *  igdb/gdl_igdb_chromosome.h 
 * 
 *  $Author: baptiste $, $Date: 2008-05-13 15:33:47 $, $Version$
 *  
 *  Libgdl : a C library for statistical genetics
 * 
 *  Copyright (C) 2009 Jean-Baptiste Veyrieras, University of Chicago, USA.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA * 
 */
#ifndef __GDL_IGDB_CHROMOSOME_H__
#define __GDL_IGDB_CHROMOSOME_H__

#include <gdl/gdl_common.h>
#include <gdl/gdl_errno.h>
#include <gdl/gdl_string.h>
#include <gdl/gdl_rng.h>
#include <gdl/gdl_vector.h>
#include <gdl/gdl_matrix.h>
#include <gdl/gdl_snp_chromosome.h>
#include <gdl/gdl_expr_chromosome.h>
#include <gdl/gdl_expr_feature.h>
#include <gdl/gdl_expr_feature_cis.h>

__BEGIN_DECLS

typedef struct
{
	size_t idx;
	gdl_string * name;
	gdl_snp_chromosome               * snp_data;
	gdl_expr_chromosome              * exp_data;
	gdl_expr_feature_chromosome      * ftr_data;
	gdl_expr_feature_cis_chromosome  * ftr_cis;
	size_t * ftr_snp_start;
	size_t * ftr_snp_end;
	size_t * ftr_snp_up;
	size_t * ftr_snp_down;
	FILE * logger;
	size_t npop;
	size_t nfeature;
	size_t nindiv;
	size_t ploidy;
	size_t * pop_sizes;
} gdl_igdb_chromosome;

gdl_igdb_chromosome * gdl_igdb_chromosome_alloc (const gdl_string * name, const size_t idx);
void gdl_igdb_chromosome_free (gdl_igdb_chromosome * c);

void gdl_igdb_chromosome_flag_snp_on_ftr_ends (gdl_igdb_chromosome * chrom);
void gdl_igdb_chromosome_flag_snp_on_ftr_window_end (gdl_igdb_chromosome * chrom, const long window);

FILE * gdl_igdb_chromosome_set_logger (gdl_igdb_chromosome * g, FILE * stream);

__END_DECLS

#endif
