/*
 *  igdb/gdl_igdb_annot.h
 *
 *  $Author: baptiste $, $Date: 2008-05-13 15:33:43 $, $Version$
 *
 *   Libgdl : a C library for statistical genetics
 *
 *  Copyright (C) 2009 Jean-Baptiste Veyrieras, University of Chicago, USA.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 */


#ifndef __GDL_IGDB_CIS_ANNOT_H__
#define __GDL_IGDB_CIS_ANNOT_H__

#include <math.h>

#include <gdl/gdl_common.h>
#include <gdl/gdl_version.h>
#include <gdl/gdl_errno.h>
#include <gdl/gdl_io.h>
#include <gdl/gdl_rng.h>
#include <gdl/gdl_randist.h>
#include <gdl/gdl_string.h>
#include <gdl/gdl_expr_feature.h>
#include <gdl/gdl_expr_feature_anchor.h>
#include <gdl/gdl_igdb_chromosome.h>
#include <gdl/gdl_igdb_genome.h>

__BEGIN_DECLS

typedef struct
{
	long start;
	long end;
	double score;
} gdl_igdb_cis_annot_entry;

gdl_igdb_cis_annot_entry * gdl_igdb_cis_annot_entry_alloc (void);
void gdl_igdb_cis_annot_entry_free (gdl_igdb_cis_annot_entry * e);

typedef struct
{
	size_t size;
	gdl_igdb_cis_annot_entry ** entries;
} gdl_igdb_cis_annot_chromosome;

gdl_igdb_cis_annot_chromosome * gdl_igdb_cis_annot_chromosome_alloc (void);
void gdl_igdb_cis_annot_chromosome_free (gdl_igdb_cis_annot_chromosome * a);
gdl_igdb_cis_annot_chromosome * gdl_igdb_cis_annot_chromosome_fscanf2 (FILE * stream);
gdl_igdb_cis_annot_chromosome * gdl_igdb_cis_annot_chromosome_fscanf3 (FILE * stream);
gdl_igdb_cis_annot_chromosome * gdl_igdb_cis_annot_chromosome_fscanf4 (FILE * stream);

typedef struct
{
	gdl_string * name;
	size_t gene_only;
	size_t spurious;
	gdl_igdb_cis_annot_chromosome * (*load)(FILE * stream);
	double (*snp_value)(const void * vtype, const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t feature_idx, const int feature_unit, const long position, const int pos_bin, const int snp_idx, size_t * is_discrete);
	long (*p_size)(const void * vtype, const gdl_igdb_cis_annot_chromosome * annot, const gdl_igdb_chromosome * chrom, const size_t gene_idx);
	double thresh;
	size_t  bin_only;
	size_t * bin_idx;
	size_t nanchor;
	gdl_expr_feature_anchor ** anchors;
	long ** anchor_ranges;
	void * extra;
} gdl_igdb_cis_annot_track_type;

gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_track_type_alloc (const gdl_igdb_cis_annot_track_type * T);
void gdl_igdb_cis_annot_track_type_free (gdl_igdb_cis_annot_track_type * t);

typedef struct
{
	gdl_igdb_cis_annot_track_type * T;
	gdl_string * name;
	gdl_string * dir;
	gdl_igdb_cis_annot_chromosome * chrom;
	//gdl_igdb_cis_annot_chromosome * focus;
	void * extra;
	// Multi-thread focus
	// (Allow multiple-thread to look at the same time at different features)
	size_t nthread;
	gdl_igdb_cis_annot_chromosome ** focusth;
} gdl_igdb_cis_annot_track;

gdl_igdb_cis_annot_track * gdl_igdb_cis_annot_track_alloc (const gdl_igdb_cis_annot_track_type * T, const gdl_string * name, const size_t nthread);
void gdl_igdb_cis_annot_track_mth (gdl_igdb_cis_annot_track * t, const size_t nthread);
void gdl_igdb_cis_annot_track_free (gdl_igdb_cis_annot_track * t);
int gdl_igdb_cis_annot_track_load (gdl_igdb_cis_annot_track * t, const gdl_igdb_chromosome * chrom);
size_t gdl_igdb_cis_annot_track_focus (const gdl_igdb_cis_annot_track * t, gdl_igdb_cis_annot_chromosome ** t_focus, const long from, const long to);
void gdl_igdb_cis_annot_track_unfocus (const gdl_igdb_cis_annot_track * t, gdl_igdb_cis_annot_chromosome ** t_focus);
double gdl_igdb_cis_annot_track_snp_value (gdl_igdb_cis_annot_track * t, const gdl_igdb_cis_annot_chromosome * t_focus, const gdl_igdb_chromosome * chrom, const size_t feature_idx, const int unit_idx, const long position, const int pos_bin, const int snp_idx, size_t * is_discrete);
long gdl_igdb_cis_annot_track_psize (gdl_igdb_cis_annot_track * t, const gdl_igdb_cis_annot_chromosome * t_focus, const gdl_igdb_chromosome * chrom, const size_t gene_idx);
int gdl_igdb_cis_annot_track_unload (gdl_igdb_cis_annot_track * t);
gdl_igdb_cis_annot_track * gdl_igdb_cis_annot_track_sscanf (const gdl_string * string);
gdl_igdb_cis_annot_track * gdl_igdb_cis_annot_track_fread (FILE * stream);
int gdl_igdb_cis_annot_track_fwrite (FILE * stream, const gdl_igdb_cis_annot_track * t);


typedef struct _gdl_igdb_cis_annot gdl_igdb_cis_annot;

struct _gdl_igdb_cis_annot
{
	size_t size;
	size_t nrep;
	size_t ** rep_idx;
	gdl_igdb_cis_annot_track ** tracks;
	size_t nnested;
	struct _gdl_igdb_cis_annot ** nested;
};

typedef struct _gdl_igdb_cis_annot gdl_igdb_cis_annotth;

struct _gdl_igdb_cis_annotth
{
	size_t size;
	size_t nrep;
	size_t ** rep_idx;
	gdl_igdb_cis_annot_track ** tracks;
	size_t nnested;
	struct _gdl_igdb_cis_annot ** nested;
};

gdl_igdb_cis_annot * gdl_igdb_cis_annot_alloc (void);
int gdl_igdb_cis_annot_nested (gdl_igdb_cis_annot * main, gdl_igdb_cis_annot * sub);
void gdl_igdb_cis_annot_unnested (gdl_igdb_cis_annot * main);
gdl_igdb_cis_annot * gdl_igdb_cis_annot_fscanf (FILE * stream);
gdl_igdb_cis_annot * gdl_igdb_cis_annot_sscanf (const gdl_string * single);
void gdl_igdb_cis_annot_mth (gdl_igdb_cis_annot * a, const size_t nthread);
void gdl_igdb_cis_annot_free (gdl_igdb_cis_annot * a);
gdl_igdb_cis_annot * gdl_igdb_cis_annot_fread (FILE * stream);
int gdl_igdb_cis_annot_fwrite (FILE * stream, const gdl_igdb_cis_annot * a);
int gdl_igdb_cis_annot_load (const gdl_igdb_cis_annot * t, const gdl_igdb_chromosome * chrom);
size_t gdl_igdb_cis_annot_focus (const gdl_igdb_cis_annot * t, const long from, const long to, const size_t thread_idx);
void gdl_igdb_cis_annot_unfocus (const gdl_igdb_cis_annot * t, const size_t thread_idx);
int gdl_igdb_cis_annot_unload (const gdl_igdb_cis_annot * t);
size_t gdl_igdb_cis_annot_size (const gdl_igdb_cis_annot * a);

GDL_VAR const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_genome_track;
GDL_VAR const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_genome_score;
GDL_VAR const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_snp_table;
GDL_VAR const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_intron;
GDL_VAR const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_cds_intron;
GDL_VAR const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_notcds_intron;
GDL_VAR const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_exon;
GDL_VAR const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_cds_exon;
GDL_VAR const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_notcds_exon;
GDL_VAR const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_first_intron;
GDL_VAR const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_last_intron;
GDL_VAR const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_first_exon;
GDL_VAR const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_last_exon;
GDL_VAR const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_5_utr;
GDL_VAR const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_3_utr;
GDL_VAR const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_inside_gene;
GDL_VAR const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_splice_site5;
GDL_VAR const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_splice_site3;
GDL_VAR const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_splice_site;
GDL_VAR const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_splice_site;
GDL_VAR const gdl_igdb_cis_annot_track_type * gdl_igdb_cis_annot_inside_feature;

#define NCBI_CODING_SYNONYMOUS "coding-synonymous"
#define NCBI_CODING_NONSYNONYMOUS "coding-nonsynonymous"

__END_DECLS

#endif
