/*
 *  igdb/gdl_igdb_genome.h
 *
 *  $Author: baptiste $, $Date: 2008-05-13 15:33:47 $, $Version$
 *
 *  Libgdl : a C library for statistical genetics
 *
 *  Copyright (C) 2009 Jean-Baptiste Veyrieras, University of Chicago, USA.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 */
#ifndef __GDL_IGDB_LOOPER_H__
#define __GDL_IGDB_LOOPER_H__

#include <gdl/gdl_common.h>
#include <gdl/gdl_errno.h>
#include <gdl/gdl_string.h>
#include <gdl/gdl_hash.h>
#include <gdl/gdl_table.h>
#include <gdl/gdl_labeled_matrix.h>
#include <gdl/gdl_snp_genome.h>
#include <gdl/gdl_expr_genome.h>
#include <gdl/gdl_expr_feature.h>
#include <gdl/gdl_expr_feature_cis.h>
#include <gdl/gdl_igdb_chromosome.h>
#include <gdl/gdl_igdb_genome.h>
#include <gdl/gdl_igdb_cis_annot.h>

__BEGIN_DECLS

typedef struct
{
	const gdl_igdb_genome       * db;
	const gdl_igdb_cis_annot    * annot;
	const gdl_feature_table     * gene_set;
	const gdl_feature_table     * gene_rm;
	const gdl_snp_table         * snp_rm;
	FILE * logger;
	long window;
	long feature_max;
	gdl_boolean unignore;
	// private
	gdl_igdb_chromosome       * _chrom;
	gdl_expr_feature          * _feature;
	gdl_expr_feature_cis      * _cis_feature;
	gdl_expr_feature_cis_snp  * _cis_snp;
	gdl_snp                   * _snp;
	gdl_snp *                 * _snps;
	size_t _feature_idx;
	size_t _thread_idx;
} gdl_igdb_cis_looper;

// multi-thread looper
typedef struct
{
	size_t  nthread;
	gdl_igdb_cis_annot   * annot;
	gdl_igdb_cis_looper ** looperth;
} gdl_igdb_cis_looperth;

gdl_igdb_cis_looper * gdl_igdb_cis_looper_alloc (const gdl_igdb_genome * db, const gdl_igdb_cis_annot * annot, const gdl_feature_table * gene_set, const gdl_feature_table * gene_rm, const gdl_snp_table * snp_rm, const long window, FILE * logger);
void gdl_igdb_cis_looper_free (gdl_igdb_cis_looper * l);
int gdl_igdb_cis_looper_init_chromosome_loop (gdl_igdb_cis_looper * hm, const size_t c);
int gdl_igdb_cis_looper_end_chromosome_loop (gdl_igdb_cis_looper * hm, const size_t c);
int gdl_igdb_cis_looper_init_feature_loop (gdl_igdb_cis_looper * hm, const size_t g);
void gdl_igdb_cis_looper_end_feature_loop (gdl_igdb_cis_looper * hm, const size_t g);
size_t gdl_igdb_cis_looper_snp_window_start (const gdl_igdb_cis_looper * hm);
size_t gdl_igdb_cis_looper_snp_window_end (const gdl_igdb_cis_looper * hm);
gdl_boolean gdl_igdb_cis_looper_snp_next (gdl_igdb_cis_looper * hm, const size_t l);
void gdl_igdb_cis_looper_snp_init (gdl_igdb_cis_looper * hm, const size_t from);

gdl_igdb_cis_looperth * gdl_igdb_cis_looperth_alloc (const size_t nthread, const gdl_igdb_genome * db, gdl_igdb_cis_annot * annot, const gdl_feature_table * gene_set, const gdl_feature_table * gene_rm, const gdl_snp_table * snp_rm, const long window);
void gdl_igdb_cis_looperth_free (gdl_igdb_cis_looperth * l);
int gdl_igdb_cis_looper_init_chromosome_loopth (gdl_igdb_cis_looperth * hm, const size_t t, const size_t c);
int gdl_igdb_cis_looper_end_chromosome_loopth (gdl_igdb_cis_looperth * hm, const size_t t, const size_t c);
int gdl_igdb_cis_looper_init_feature_loopth (gdl_igdb_cis_looperth * hm, const size_t t, const size_t g);
void gdl_igdb_cis_looper_end_feature_loopth (gdl_igdb_cis_looperth * hm, const size_t t, const size_t g);
size_t gdl_igdb_cis_looper_snp_window_startth (const gdl_igdb_cis_looperth * hm, const size_t t);
size_t gdl_igdb_cis_looper_snp_window_endth (const gdl_igdb_cis_looperth * hm, const size_t t);
gdl_boolean gdl_igdb_cis_looper_snp_nextth (gdl_igdb_cis_looperth * hm, const size_t t, const size_t l);

__END_DECLS

#endif
