/*
 *  igdb/gdl_igdb_genome.h
 *
 *  $Author: baptiste $, $Date: 2008-05-13 15:33:47 $, $Version$
 *
 *  Libgdl : a C library for statistical genetics
 *
 *  Copyright (C) 2009 Jean-Baptiste Veyrieras, University of Chicago, USA.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
 */
#ifndef __GDL_IGDB_GENOME_H__
#define __GDL_IGDB_GENOME_H__

#include <gdl/gdl_common.h>
#include <gdl/gdl_errno.h>
#include <gdl/gdl_string.h>
#include <gdl/gdl_hash.h>
#include <gdl/gdl_table.h>
#include <gdl/gdl_labeled_matrix.h>
#include <gdl/gdl_snp_genome.h>
#include <gdl/gdl_expr_genome.h>
#include <gdl/gdl_expr_feature.h>
#include <gdl/gdl_expr_feature_cis.h>
#include <gdl/gdl_igdb_chromosome.h>

__BEGIN_DECLS

typedef struct
{
	size_t nchrom;
	gdl_string ** chroms;
	gdl_string  * dbdir;
	gdl_snp_genome     * s_genome;
	gdl_expr_genome    * e_genome;
	gdl_expr_feature_genome     * f_genome;
	gdl_expr_feature_cis_genome * r_genome;
	gdl_table        * sample_info;
	FILE * logger;
	size_t npop;
	gdl_string ** pop_names;
} gdl_igdb_genome;

gdl_igdb_genome * gdl_igdb_genome_alloc (const gdl_string * config_file);
void gdl_igdb_genome_free (gdl_igdb_genome * v);

gdl_igdb_chromosome * gdl_igdb_genome_get (const gdl_igdb_genome * g, size_t i);
gdl_igdb_chromosome * gdl_igdb_genome_lookup (const gdl_igdb_genome * g, const gdl_string * name);
int gdl_igdb_genome_set (const gdl_igdb_genome * g, size_t i, gdl_igdb_chromosome * c);
gdl_igdb_genome * gdl_igdb_genome_fread (FILE * stream);
int gdl_igdb_genome_fwrite (FILE * stream, const gdl_igdb_genome * v);
int gdl_igdb_genome_summary (FILE * stream, const gdl_igdb_genome * v);
FILE * gdl_igdb_genome_set_logger (gdl_igdb_genome * g, FILE * stream);

int gdl_igdb_genome_load_feature (gdl_igdb_genome * g, const gdl_string * feature_file);
int gdl_igdb_genome_load_cis_feature (gdl_igdb_genome * g, const gdl_string * cis_feature_file);

gdl_labeled_matrix * gdl_igdb_get_fexp_matrix (const gdl_igdb_genome * g, const gdl_boolean qqnorm, const gdl_boolean center, gdl_feature_table * feature_table);
gdl_igdb_genome * gdl_igdb_genome_subset (gdl_igdb_genome * g, gdl_string ** sample_names, gdl_string ** sub_sample_names, const size_t nsub_sample, const gdl_string * output);
int gdl_igdb_genome_fprintf (gdl_igdb_genome * g, const gdl_string * output);

__END_DECLS

#endif
