/*
 *  igdb/gdl_igdb_snp_reg.h 
 * 
 *  $Author: baptiste $, $Date: 2008-05-13 15:33:47 $, $Version$
 *  
 *  Libgdl : a C library for statistical genetics
 * 
 *  Copyright (C) 2009 Jean-Baptiste Veyrieras, University of Chicago, USA.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA * 
 */
#ifndef __GDL_IGDB_SNP_REG_H__
#define __GDL_IGDB_SNP_REG_H__ 
 
#include <math.h>

#include <gdl/gdl_common.h>
#include <gdl/gdl_errno.h>
#include <gdl/gdl_string.h>
#include <gdl/gdl_matrix.h>
#include <gdl/gdl_labeled_matrix.h>
#include <gdl/gdl_vector.h>
#include <gdl/gdl_blas.h>
#include <gdl/gdl_cblas.h>
#include <gdl/gdl_math.h>
#include <gdl/gdl_rng.h>
#include <gdl/gdl_randist.h>
#include <gdl/gdl_specfunc.h>
#include <gdl/gdl_sort_double.h>
#include <gdl/gdl_statistics_double.h>
#include <gdl/gdl_bayesian_regression.h>
#include <gdl/gdl_snp_annotation.h>
#include <gdl/gdl_igdb_chromosome.h>

__BEGIN_DECLS

typedef struct
{
	gdl_string * chrom_name;
	gdl_string * snp_name;
	long snp_pos;
	double snp_maf;
	size_t N;
	double a;
	double r2;
	double pval;
	double bf;
} gdl_igdb_reg_snp_record;

gdl_igdb_reg_snp_record * gdl_igdb_reg_snp_record_alloc (const gdl_string * chrom_name, const gdl_string * snp_name, const long snp_pos, double snp_maf, const size_t N, const double a, const double r2, const double pval, const double bf);
gdl_igdb_reg_snp_record * gdl_igdb_reg_snp_record_alloc2 (void);
void gdl_igdb_reg_snp_record_update (gdl_igdb_reg_snp_record * r, const gdl_string * chrom_name, const gdl_string * snp_name, const long snp_pos, double snp_maf, const size_t N, const double a, const double r2, const double pval, const double bf);
void gdl_igdb_reg_snp_record_free (gdl_igdb_reg_snp_record * r);
int gdl_igdb_reg_snp_record_fprintf (FILE * stream, const gdl_igdb_reg_snp_record * r);

typedef struct
{
	const gdl_feature_table * feature_table;
	const gdl_labeled_matrix * Y;
	size_t from;
	size_t to;
	size_t size;
	size_t * nbf;
	size_t * npv;
	double * bf_max;
	double * pv_min;
	gdl_igdb_reg_snp_record *** bf_records;
	gdl_igdb_reg_snp_record *** pv_records;
} gdl_igdb_reg_best_buffer;

gdl_igdb_reg_best_buffer * gdl_igdb_reg_best_buffer_alloc (const gdl_labeled_matrix * Y, const int y_from, const int y_to, const gdl_feature_table * feature_table);
void gdl_igdb_reg_best_buffer_free (gdl_igdb_reg_best_buffer * b);
int gdl_igdb_reg_best_buffer_fprintf (FILE * stream, const gdl_igdb_reg_best_buffer * b, const unsigned char mode);
gdl_igdb_reg_snp_record * gdl_igdb_reg_best_buffer_test_pv (gdl_igdb_reg_best_buffer * b, const size_t fx, const double pv, const unsigned char mode);
gdl_igdb_reg_snp_record * gdl_igdb_reg_best_buffer_test_bf (gdl_igdb_reg_best_buffer * b, const size_t fx, const double bf, const unsigned char mode);

// Global
size_t gdl_igdb_reg_snp_init (gdl_bayreg_workspace * bayes, const gdl_snp_chromosome * chrom, const size_t N, const size_t snp, double popf[], double * f, size_t * rm_individual[]);
double gdl_igdb_reg_perform_snp_add (gdl_bayreg_workspace * bayes, double * a, double * r2, double * p);
void gdl_igdb_reg_permut (gdl_bayreg_workspace * bayes, gdl_igdb_chromosome * chrom, const gdl_rng * rng, const size_t within);
void gdl_igdb_reg_feature_init (gdl_bayreg_workspace * bayes, const gdl_igdb_chromosome * chrom, gdl_matrix * Y, const size_t yi, const gdl_boolean qnorm);

// Full regression
int gdl_igdb_chromosome_reg_run (gdl_igdb_chromosome * chrom, gdl_bayreg_model * model, const gdl_boolean qnorm, size_t * rm_individual[], const gdl_feature_table * feature_table, const gdl_labeled_matrix * Y, const int y_from, const int y_to, gdl_igdb_reg_best_buffer * best, const double bf_min, const double pv_max, FILE * stream);
int gdl_igdb_chromosome_reg_permut (gdl_igdb_chromosome * chrom, gdl_bayreg_model * model, const gdl_boolean qnorm, size_t * rm_individual[], const gdl_feature_table * feature_table, const gdl_labeled_matrix * Y, const int y_from, const int y_to, const gdl_rng * rng, const size_t permut, gdl_igdb_reg_best_buffer * best, const double bf_min, const double pv_max, FILE * stream);
// Cis regression
void gdl_igdb_cis_reg_feature_init (gdl_bayreg_workspace * bayes, gdl_igdb_chromosome * chrom, gdl_expr_feature * feature, const gdl_boolean qnorm);
gdl_expr_feature_cis_chromosome * gdl_igdb_chromosome_cis_reg_run (gdl_igdb_chromosome * chrom, gdl_bayreg_model * model, const gdl_boolean qnorm, size_t * rm_individual[], const gdl_feature_table * feature_set);
int gdl_igdb_chromosome_cis_reg_partial_run (gdl_igdb_chromosome * chrom, gdl_bayreg_model * model, gdl_expr_feature_cis_chromosome * storage, const size_t from, const size_t to, const gdl_boolean qnorm, size_t * rm_individual[], const gdl_feature_table * feature_set);
int gdl_igdb_chromosome_cis_freq_permut (gdl_igdb_chromosome * chrom, const gdl_boolean qnorm, const gdl_rng * rng, const size_t permut, const gdl_boolean best_snp, FILE * stream);
int gdl_igdb_chromosome_cis_bayes_permut (gdl_igdb_chromosome * chrom, gdl_bayreg_model * model, const gdl_boolean qnorm, const gdl_rng * rng, const size_t permut, const gdl_boolean best_snp, FILE * stream);
int gdl_igdb_chromosome_cis_freq_partial_permut (gdl_igdb_chromosome * chrom, const size_t from, const size_t to, const gdl_boolean qnorm, const gdl_rng * rng, const size_t permut, const gdl_boolean best_snp, FILE * stream);
int gdl_igdb_chromosome_cis_bayes_partial_permut (gdl_igdb_chromosome * chrom, gdl_bayreg_model * model, const size_t from, const size_t to, const gdl_boolean qnorm, const gdl_rng * rng, const size_t permut, const gdl_boolean best_snp, FILE * stream);
int gdl_igdb_cis_reg_residuals (gdl_bayreg_model * model, gdl_bayreg_workspace * work, gdl_igdb_chromosome * chrom, gdl_expr_feature * feature, const size_t sx);

__END_DECLS

#endif
